import pytest
from collections import OrderedDict

from arctic.mixins import FormMixin
from articles.forms import ArticleForm

from tests.conftest import get_form
from tests.factories import ArticleFactory


@pytest.fixture
def layout():
    class Layout(FormMixin):
        layout = None

        def __init__(self):
            self.object = ArticleFactory()
            self.form = ArticleForm(instance=self.object)
            self.get_form = get_form(self.form)

    return Layout()


pytestmark = pytest.mark.django_db


def test_layout_example_1(layout):
    layout.layout = ['title|8']
    layout = layout.get_layout()

    assert layout[0]['fieldset']['title'] is None
    assert layout[0]['fieldset']['description'] is None
    assert layout[0]['fieldset']['collapsible'] is None
    assert layout[0]['rows'][0]['name'] == 'title'
    assert layout[0]['rows'][0]['column'] == '8'


def test_layout_example_2(layout):
    layout.layout = [['title|3', 'title', 'title']]
    layout = layout.get_layout()

    assert layout[0]['fieldset']['title'] is None
    assert layout[0]['fieldset']['description'] is None
    assert layout[0]['fieldset']['collapsible'] is None
    assert layout[0]['rows'][0][0]['name'] == 'title'
    assert layout[0]['rows'][0][0]['column'] == '3'
    assert layout[0]['rows'][0][1]['name'] == 'title'
    assert layout[0]['rows'][0][1]['column'] == '4'
    assert layout[0]['rows'][0][2]['name'] == 'title'
    assert layout[0]['rows'][0][2]['column'] == '5'


def test_layout_example_3a(layout):
    layout.layout = [['title|3', 'title', 'title', 'category', 'category']]
    layout = layout.get_layout()

    assert layout[0]['fieldset']['title'] is None
    assert layout[0]['fieldset']['description'] is None
    assert layout[0]['fieldset']['collapsible'] is None
    assert layout[0]['rows'][0][0]['name'] == 'title'
    assert layout[0]['rows'][0][0]['column'] == '3'
    assert layout[0]['rows'][0][1]['name'] == 'title'
    assert layout[0]['rows'][0][1]['column'] == '2'
    assert layout[0]['rows'][0][2]['name'] == 'title'
    assert layout[0]['rows'][0][2]['column'] == '2'
    assert layout[0]['rows'][0][3]['name'] == 'category'
    assert layout[0]['rows'][0][3]['column'] == '2'
    assert layout[0]['rows'][0][4]['name'] == 'category'
    assert layout[0]['rows'][0][4]['column'] == '3'


def test_layout_example_3b(layout):
    layout.layout = ['title|3', 'title', 'title', ['category', 'category']]
    layout = layout.get_layout()

    assert layout[0]['fieldset']['title'] is None
    assert layout[0]['fieldset']['description'] is None
    assert layout[0]['fieldset']['collapsible'] is None
    assert layout[0]['rows'][0]['name'] == 'title'
    assert layout[0]['rows'][0]['column'] == '3'
    assert layout[0]['rows'][1]['name'] == 'title'
    assert layout[0]['rows'][1]['column'] is None
    assert layout[0]['rows'][2]['name'] == 'title'
    assert layout[0]['rows'][2]['column'] is None
    assert layout[0]['rows'][3][0]['name'] == 'category'
    assert layout[0]['rows'][3][0]['column'] == '6'
    assert layout[0]['rows'][3][1]['name'] == 'category'
    assert layout[0]['rows'][3][1]['column'] == '6'


def test_layout_example_4(layout):
    layout.layout = OrderedDict([('-fieldset',
                                  ['title',
                                   'title',
                                   ['category', 'updated_at|4']]),
                                ('fieldset2|test description',
                                 [['title|7', 'category']]),
                                ('+fieldset3',
                                 ['published'])])
    layout = layout.get_layout()

    assert layout[0]['fieldset']['title'] == 'fieldset'
    assert layout[0]['fieldset']['description'] is None
    assert layout[0]['fieldset']['collapsible'] == 'closed'
    assert layout[0]['rows'][0]['name'] == 'title'
    assert layout[0]['rows'][0]['column'] is None
    assert layout[0]['rows'][1]['name'] == 'title'
    assert layout[0]['rows'][1]['column'] is None
    assert layout[0]['rows'][2][0]['name'] == 'category'
    assert layout[0]['rows'][2][0]['column'] == '8'
    assert layout[0]['rows'][2][1]['name'] == 'updated_at'
    assert layout[0]['rows'][2][1]['column'] == '4'

    assert layout[1]['fieldset']['title'] == 'fieldset2'
    assert layout[1]['fieldset']['description'] == 'test description'
    assert layout[1]['fieldset']['collapsible'] is None
    assert layout[1]['rows'][0][0]['name'] == 'title'
    assert layout[1]['rows'][0][0]['column'] == '7'
    assert layout[1]['rows'][0][1]['name'] == 'category'
    assert layout[1]['rows'][0][1]['column'] == '5'

    assert layout[2]['fieldset']['title'] == 'fieldset3'
    assert layout[2]['fieldset']['description'] is None
    assert layout[2]['fieldset']['collapsible'] == 'open'
    assert layout[2]['rows'][0]['name'] == 'published'
    assert layout[2]['rows'][0]['column'] is None



import pytest

from arctic.models import Role
from django.urls import reverse

from tests.factories import UserFactory


@pytest.mark.django_db
class TestUserUpdate(object):

    def get_url(self, pk):
        return reverse('users:detail', kwargs={'pk': pk})

    def test_update_page(self, admin_client):
        user = UserFactory()

        response = admin_client.get(self.get_url(user.pk))
        assert response.status_code == 200

    def test_user_submit(self, django_user_model, admin_client):
        user = UserFactory(username='u1', email='u1@test.test')

        username = 'u1-changed'
        email = 'u1-changed@test.test'

        response = admin_client.post(
            self.get_url(user.pk),
            {'user-email': email,
             'role-role': str(Role.objects.get(name='editor').pk),
             'user-username': username, }
        )

        assert response.status_code == 302
        user = django_user_model.objects.get(pk=user.pk)
        assert user.email == email
        assert user.username == username



"""
Django settings for {{ project_name }} project.

Generated by 'arctic start' command based on Django 2.2

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "(nb89xnt7&ez3%6c4djx+e^!wi+d=h6qyq(14k7x_ipcmr_464"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

# Application definition

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "config",
    "arctic",
    "dashboard",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "config.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

WSGI_APPLICATION = "config.wsgi.application"

# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

# Password validation
# https://docs.djangoproject.com/en/2.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"  # NOQA
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator" # NOQA
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator" # NOQA
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator" # NOQA
    },
]

# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = "/static/"

LOGIN_URL = "login"
LOGOUT_URL = "logout"

# Arctic configuration
ARCTIC_SITE_NAME = "{{ project_name }}'s Arctic website"

ARCTIC_MENU = (
        ("Dashboard", "index", "fa-dashboard"),
        )



from __future__ import absolute_import, unicode_literals

import importlib
from collections import OrderedDict
from copy import deepcopy

from django.conf import settings
from django.core.exceptions import FieldDoesNotExist, ImproperlyConfigured
from django.template.defaultfilters import slugify
from django.urls import (
    NoReverseMatch,
    get_ns_resolver,
    get_resolver,
    get_urlconf,
    reverse,
)
from django.utils import translation

from . import defaults


def is_active(menu_entry, url_name):
    if url_name:
        if menu_entry[1] == url_name:
            return True
        elif len(menu_entry) >= 3:
            related_urls = []
            related_submenus = []

            for menu_item in menu_entry[2:]:
                if is_list_of_list(menu_item):
                    # suppose it is submenus
                    related_submenus = menu_item
                elif isinstance(menu_item, (tuple, list)):
                    # suppose it is related urls names
                    related_urls += list(menu_item)
            related_urls += [submenu[1] for submenu in related_submenus]
            if url_name in related_urls:
                return True
            # Check for * in the related url's
            start_url = []
            for related in related_urls:
                if related.endswith("*"):
                    start_url += [related]
            if len(start_url):
                url_base_name = url_name.split(":")
                del url_base_name[-1]
                url_base_name = ":".join(url_base_name)
                for item in start_url:
                    return item.startswith(url_base_name)
    return False


# TODO: menu needs to hide entries not available to a certain user role
# by getting the view class from the named url we can check which permissions
# are available:
# http://stackoverflow.com/questions/5749075


def menu(menu_config=None, **kwargs):
    """
    Tranforms a menu definition into a dictionary which is a frendlier format
    to parse in a template.
    """
    request = kwargs.pop("request", None)
    user = kwargs.pop("user", None)
    url_full_name = ":".join(
        [request.resolver_match.namespace, request.resolver_match.url_name]
    )

    if not menu_config:
        menu_config = settings.ARCTIC_MENU

    menu_dict = OrderedDict()
    for menu_entry in menu_config:
        if type(menu_entry) in (list, tuple):

            # check permission based on named_url
            path = None
            if menu_entry[1]:
                if not view_from_url(menu_entry[1]).has_permission(user):
                    continue

                path = reverse(menu_entry[1])
            # icons and collapse are optional
            icon = None
            if (len(menu_entry) >= 3) and (
                not type(menu_entry[2]) in (list, tuple)
            ):
                icon = menu_entry[2]
            active_weight = len(path) if path else 0
            menu_dict[menu_entry[0]] = {
                "url": menu_entry[1],
                "icon": icon,
                "submenu": None,
                "active": is_active(menu_entry, url_full_name),
                "active_weight": active_weight,
            }

            # check if the last item in a menu entry is a submenu
            submenu = _get_submenu(menu_entry)
            if submenu:
                menu_dict[menu_entry[0]]["submenu"] = menu(
                    submenu, user=user, request=request
                )
    return menu_clean(menu_dict)


def _get_submenu(menu_entry):
    for item in menu_entry[-2:]:
        if is_list_of_list(item):
            return item


def menu_clean(menu_config):
    """
    Make sure that only the menu item with the largest weight is active.
    If a child of a menu item is active, the parent should be active too.
    :param menu:
    :return:
    """
    max_weight = -1
    for _, value in list(menu_config.items()):
        if value["submenu"]:
            for _, v in list(value["submenu"].items()):
                if v["active"]:
                    # parent inherits the weight of the axctive child
                    value["active"] = True
                    value["active_weight"] = v["active_weight"]

        if value["active"]:
            max_weight = max(value["active_weight"], max_weight)

    if max_weight > 0:
        # one of the items is active: make items with lesser weight inactive
        for _, value in list(menu_config.items()):
            if value["active"] and value["active_weight"] < max_weight:
                value["active"] = False
    return menu_config


def view_from_url(named_url):  # noqa
    """
    Finds and returns the view class from a named url
    """
    # code below is `stolen` from django's reverse method.
    resolver = get_resolver(get_urlconf())

    if type(named_url) in (list, tuple):
        named_url = named_url[0]
    parts = named_url.split(":")
    parts.reverse()
    view = parts[0]
    path = parts[1:]
    current_path = None
    resolved_path = []
    ns_pattern = ""
    ns_converters = {}

    # if it's a local url permission already given, so we just return true
    if named_url.startswith("#"):

        class LocalUrlDummyView:
            @staticmethod
            def has_permission(user):
                return True

        return LocalUrlDummyView

    while path:
        ns = path.pop()
        current_ns = current_path.pop() if current_path else None

        # Lookup the name to see if it could be an app identifier
        try:
            app_list = resolver.app_dict[ns]
            # Yes! Path part matches an app in the current Resolver
            if current_ns and current_ns in app_list:
                # If we are reversing for a particular app,
                # use that namespace
                ns = current_ns
            elif ns not in app_list:
                # The name isn't shared by one of the instances
                # (i.e., the default) so just pick the first instance
                # as the default.
                ns = app_list[0]
        except KeyError:
            pass

        if ns != current_ns:
            current_path = None

        try:
            extra, resolver = resolver.namespace_dict[ns]
            resolved_path.append(ns)
            ns_pattern = ns_pattern + extra
            try:
                ns_converters.update(resolver.pattern.converters)
            except Exception:
                pass
        except KeyError as key:
            if resolved_path:
                raise NoReverseMatch(
                    "%s is not a registered namespace inside '%s'"
                    % (key, ":".join(resolved_path))
                )
            else:
                raise NoReverseMatch("%s is not a registered namespace" % key)
    if ns_pattern:
        try:
            resolver = get_ns_resolver(
                ns_pattern, resolver, tuple(ns_converters.items())
            )
        except Exception:
            resolver = get_ns_resolver(ns_pattern, resolver)

    # custom code, get view from reverse_dict
    reverse_dict = resolver.reverse_dict.dict()
    for key, url_obj in reverse_dict.items():
        if url_obj == reverse_dict[view] and key != view:
            module = importlib.import_module(key.__module__)
            return getattr(module, key.__name__)


def find_attribute(obj, value):
    """
    Finds the attribute connected to the last object when a chain of
    connected objects is given in a string separated with double underscores.
    For example when a model x has a foreign key to model y and model y has
    attribute a, findattr(x, 'y__a') will return the a attribute from the y
    model that exists in x.
    """
    if "__" in value:
        value_list = value.split("__")
        attr = get_attribute(obj, value_list[0])
        return find_attribute(attr, "__".join(value_list[1:]))
    return get_attribute(obj, value)


def get_attribute(obj, value):
    """
    Normally the result of list_items for listviews are a set of model objects.
    But when you want a GROUP_BY query (with 'values' method), than
    the result will be a dict. This method will help you find an item for
    either objects or dictionaries.
    """
    if type(obj) == dict:
        return dict.get(obj, value)
    else:
        return getattr(obj, value)


def find_field_meta(obj, value):
    """
    In a model, finds the attribute meta connected to the last object when
    a chain of  connected objects is given in a string separated with double
    underscores.
    """
    if "__" in value:
        value_list = value.split("__")
        child_obj = obj._meta.get_field(value_list[0]).rel.to
        return find_field_meta(child_obj, "__".join(value_list[1:]))
    return obj._meta.get_field(value)


def get_field_class(qs, field_name):
    """
    Given a queryset and a field name, it will return the field's class
    """
    try:
        return qs.model._meta.get_field(field_name).__class__.__name__
    # while annotating, it's possible that field does not exists.
    except FieldDoesNotExist:
        return None


def reverse_url(url, obj, fallback_field=None):
    """
    Reverses a named url, in addition to the standard django reverse, it also
    accepts a list of ('named url', 'field1', 'field2', ...) and will use the
    value of the supplied fields as arguments.
    When a fallback field is given it will use it as an argument if none other
    are given.
    """
    args = []
    if type(url) in (list, tuple):
        named_url = url[0]
        for arg in url[1:]:
            if type(obj) is dict:
                args.append(obj[arg])
            else:
                args.append(find_attribute(obj, arg))
    else:
        if url[0] in "#/":  # local url or a path, just return it
            return url
        named_url = url
        if obj and fallback_field:
            if type(obj) is dict:
                args = [obj[fallback_field]]
            else:
                args = [get_attribute(obj, fallback_field)]

    # Instead of giving NoReverseMatch exception it's more desirable,
    # for field_links in listviews to just ignore the link.
    if fallback_field and not args:
        return ""

    return reverse(named_url, args=args)


def arctic_setting(setting_name, valid_options=None):
    """
    Tries to get a setting from the django settings, if not available defaults
    to the one defined in defaults.py
    """
    try:
        value = getattr(settings, setting_name)
        if valid_options and value not in valid_options:
            error_message = "Invalid value for {}, must be one of: {}".format(
                setting_name, str(valid_options)
            )
            raise ImproperlyConfigured(error_message)
    except AttributeError:
        pass
    return getattr(settings, setting_name, getattr(defaults, setting_name))


class RemoteDataSet:
    url_template = "?{filters}{fields}{order}{paginate}"
    paginate_template = "&offset={}&limit={}"
    order_separator = ","
    order_template = "&order={}"
    fields_separator = ","
    fields_template = "&fields={}"
    filters_template = "{}"
    filters_template_kv = "&{key}={value}"
    count = -1
    _options = {"fields": "", "order": "", "filters": "", "paginate": ""}
    page_size = None

    def fields(self, fields):
        if fields:
            fields_str = self.fields_separator.join(fields)
            self._options["fields"] = self.fields_template.format(fields_str)
        return deepcopy(self)

    def order_by(self, order):
        if order:
            order_str = self.order_separator.join(order)
            self._options["order"] = self.order_template.format(order_str)
        return deepcopy(self)

    def filter(self, **kwargs):
        if kwargs:
            filters = ""
            for key, value in kwargs.items():
                filters += self.filters_template_kv.format(
                    key=key, value=value
                )
            self._options["filters"] = self.filters_template.format(filters)
        return deepcopy(self)

    def get_url(self, start, stop):
        self._options["paginate"] = self.paginate_template.format(start, stop)
        url = self.url_template.format(**self._options)
        return url.replace("?&", "?")

    def get(self, start, stop):
        pass

    def __len__(self):
        return self.count

    def __getitem__(self, slice):
        return self.get(slice.start, slice.stop)


def offset_limit(func):
    """
    Decorator that converts python slicing to offset and limit
    """

    def func_wrapper(self, start, stop):
        offset = start
        limit = stop - start
        return func(self, offset, limit)

    return func_wrapper


def is_list_of_list(item):
    """
    check whether the item is list (tuple)
    and consist of list (tuple) elements
    """
    if (
        type(item) in (list, tuple)
        and len(item)
        and isinstance(item[0], (list, tuple))
    ):
        return True
    return False


def generate_id(*s):
    """
    generates an id from one or more given strings
    it uses english as the base language in case some strings
    are translated, this ensures consistent ids
    """
    with translation.override("en"):
        generated_id = slugify("-".join([str(i) for i in s]))
    return generated_id


def append_query_parameter(url, parameters, ignore_if_exists=True):
    """ quick and dirty appending of query parameters to a url """
    if ignore_if_exists:
        keys = list(parameters.keys())
        for key in keys:
            if key + "=" in url:
                del parameters[key]
    parameters_str = "&".join(k + "=" + v for k, v in parameters.items())
    append_token = "&" if "?" in url else "?"
    return url + append_token + parameters_str



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-10-10 13:54
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, unique=True, verbose_name='Role')),
                ('is_active', models.BooleanField(default=True)),
            ],
            options={
                'swappable': 'ARCTIC_ROLE_MODEL',
            },
        ),
        migrations.CreateModel(
            name='UserRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('role', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=getattr(settings, 'ARCTIC_ROLE_MODEL', 'arctic.Role'))),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='urole', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'swappable': 'ARCTIC_USER_ROLE_MODEL',
            },
        ),
    ]



from django.urls import reverse, reverse_lazy
from django.utils.translation import gettext_lazy as _

from arctic.generics import (
    CreateView,
    DeleteView,
    ListView,
    UpdateView,
)

from .forms import {{ camel_case_app_name }}Form
from .models import {{ camel_case_app_name }}


class {{ camel_case_app_name }}ListView(ListView):
    model = {{ camel_case_app_name }}
    fields = '__all__'
    permission_required = 'view_{{ app_name }}'

    # Delete and detail action link
    action_links = [
        ("detail", "{{ app_name}}:detail", "fa-edit"),
        ("delete", "{{ app_name}}:delete", "fa-trash"),
    ]

    # tool link to create
    tool_links = [
        (_("Create {{ app_name }}"), "{{ app_name }}:create", "fa-plus"),
    ]

    # Some optional fields
    # paginate_by = 10
    # ordering_fields = ['field_name1', ..]
    # search_fields = ['field_name', ...]
    # allowed_exports = ["csv"]


class {{ camel_case_app_name }}CreateView(CreateView):
    model = {{ camel_case_app_name }}
    form_class = {{ camel_case_app_name }}Form
    permission_required = 'add_{{ app_name }}'

    def get_success_url(self):
        return reverse("{{app_name}}:detail", args=(self.object.pk,))


class {{ camel_case_app_name }}UpdateView(UpdateView):
    model = {{ camel_case_app_name }}
    form_class = {{ camel_case_app_name }}Form
    permission_required = 'change_{{ app_name }}'
    success_url = reverse_lazy('{{app_name}}:list')
    actions = [
        (_("Cancel"), "cancel"),
        (_("Save"), "submit"),
    ]


class {{ camel_case_app_name }}DeleteView(DeleteView):
    model = {{ camel_case_app_name }}
    success_url = reverse_lazy('{{app_name}}:list')
    permission_required = 'delete_{{ app_name }}'



from collections import OrderedDict

from django import forms
from django.contrib.auth import forms as user_forms
from django.contrib.auth import get_user_model
from django.utils.translation import gettext as _

from arctic.loading import get_user_role_model
from arctic.multiform import MultiModelForm

User = get_user_model()
UserRole = get_user_role_model()

# it would be even nicer to raise an error here to explicitly
# ask user to set these properties but we need backwards compatibility
DEFAULT_FIELDS = (User.USERNAME_FIELD, "email", "is_active")
FIELDS_CREATE = getattr(User, "FIELDS_CREATE", DEFAULT_FIELDS)
FIELDS_UPDATE = getattr(User, "FIELDS_UPDATE", DEFAULT_FIELDS)


class UserRoleForm(forms.ModelForm):
    class Meta:
        model = UserRole
        fields = ("role",)


class UserCreationForm(user_forms.UserCreationForm):
    class Meta:
        model = User
        fields = FIELDS_CREATE


class UserCreationMultiForm(MultiModelForm):
    form_classes = OrderedDict(
        [("user", UserCreationForm), ("role", UserRoleForm)]
    )

    def save(self, commit=True):
        objects = super(UserCreationMultiForm, self).save(commit=False)

        if commit:
            user = objects["user"]
            user.save()
            user_role = objects["role"]
            user_role.user = user
            user_role.save()

        return objects


class UserChangeForm(forms.ModelForm):
    new_password = forms.CharField(
        label=_("New Password"),
        widget=forms.PasswordInput,
        required=False,
        help_text=_(
            "Leave this field empty if you don't want to change your "
            "password."
        ),
    )

    class Meta:
        model = User
        fields = FIELDS_UPDATE

    def save(self, commit=True):
        user = super(UserChangeForm, self).save(commit=False)

        new_password = self.cleaned_data["new_password"]
        if new_password:
            user.set_password(new_password)
        if commit:
            user.save()
        return user


class UserChangeMultiForm(MultiModelForm):
    form_classes = OrderedDict(
        [("user", UserChangeForm), ("role", UserRoleForm)]
    )



from __future__ import absolute_import, unicode_literals

from arctic.forms import SimpleSearchForm
from arctic.widgets import QuickFiltersSelect
from django import forms
from django.db.models import Q

from .models import Article


class ArticleForm(forms.ModelForm):
    class Meta:
        fields = [
            "title",
            "description",
            "category",
            "updated_at",
            "tags",
            "published",
        ]
        model = Article

    def __init__(self, *args, **kwargs):
        super(ArticleForm, self).__init__(*args, **kwargs)
        self.fields["category"].empty_label = ""


class AdvancedArticleSearchForm(forms.Form):
    description = forms.CharField(
        max_length=100, required=False, label="Description"
    )

    def __init__(self, data):
        # Reset data, but store for get_search_filter
        self.stored_data = data
        super(AdvancedArticleSearchForm, self).__init__(data)

    def get_search_filter(self):
        value = self.cleaned_data.get("description")
        if value:
            return Q(description__icontains=value)
        return Q()


class FiltersAndSearchForm(SimpleSearchForm):
    field_order = ["published", "search"]
    FILTER_CHOICES = (("published", "Published"), ("drafts", "Drafts"))
    published = forms.ChoiceField(
        choices=FILTER_CHOICES,
        widget=QuickFiltersSelect(attrs={"class": "spacer", "submit": True}),
        required=False,
    )
    # date = forms.CharField(required=False,
    #                        widget=DateTimePickerInput(
    #                            attrs={'placeholder': 'Start date'}))

    def get_search_filter(self):
        q = super(FiltersAndSearchForm, self).get_search_filter()

        published_value = self.cleaned_data.get("published")
        if published_value in ("published", "drafts"):
            published = True if published_value == "published" else False

            q &= Q(published=published)
        return q



# Generated by Django 2.0.6 on 2018-06-18 08:45

import articles.models
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('articles', '0003_category_order'),
    ]

    operations = [
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=255, null=True)),
                ('image', models.ImageField(upload_to=articles.models.filename)),
                ('order', models.IntegerField(blank=True, null=True)),
                ('article', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='articles.Article')),
            ],
            options={
                'ordering': ['order'],
            },
        ),
    ]



from __future__ import division, unicode_literals

import calendar
import csv
import json
from collections import OrderedDict

from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth.views import SuccessURLAllowedHostsMixin
from django.core.exceptions import (
    FieldDoesNotExist,
    ImproperlyConfigured,
    ValidationError,
)
from django.core.paginator import InvalidPage
from django.db.models.deletion import Collector, ProtectedError
from django.db.models.manager import Manager
from django.forms.widgets import Media
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import redirect, render, resolve_url
from django.urls import NoReverseMatch, reverse
from django.utils.formats import get_format
from django.utils.html import mark_safe, strip_tags
from django.utils.http import quote
from django.utils.text import capfirst
from django.utils.translation import get_language
from django.utils.translation import gettext as _
from django.views import generic as base

import extra_views

from .mixins import (
    FormMediaMixin,
    FormMixin,
    ListMixin,
    RoleAuthentication,
    SuccessMessageMixin,
)
from .paginator import IndefinitePaginator
from .utils import (
    append_query_parameter,
    arctic_setting,
    find_attribute,
    find_field_meta,
    get_field_class,
    menu,
    reverse_url,
    view_from_url,
)

try:
    from django.utils.http import url_has_allowed_host_and_scheme
except ImportError:
    from django.utils.http import (
        is_safe_url as url_has_allowed_host_and_scheme,
    )


def collapsible(fiedset_name, collapsed=False):
    """ Utility method for Form layouts """
    return "{}{}".format("-" if collapsed else "+", fiedset_name)


def collapsible_gettext(fiedset_name, collapsed=False):
    return collapsible(_(fiedset_name), collapsed)


class View(RoleAuthentication, base.View):
    """
    This view needs to be used for all Arctic views, except the LoginView.

    It includes integration with the Arctic user interface elements, such as
    the menu, site logo, site title, page title and breadcrumbs.
    """

    page_title = ""
    page_description = ""
    breadcrumbs = None
    tabs = None
    requires_login = True
    urls = {}
    form_display = None
    in_modal = False

    def dispatch(self, request, *args, **kwargs):
        """
        Most views in a CMS require a login, so this is the default setup.

        If a login is not required then the requires_login property
        can be set to False to disable this.
        """
        if self.requires_login:
            if settings.LOGIN_URL is None or settings.LOGOUT_URL is None:
                raise ImproperlyConfigured(
                    "LOGIN_URL and LOGOUT_URL "
                    "has to be defined if requires_login is True"
                )

            if not request.user.is_authenticated:
                return redirect(
                    "%s?next=%s"
                    % (
                        resolve_url(settings.LOGIN_URL),
                        quote(request.get_full_path()),
                    )
                )

        return super(View, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(View, self).get_context_data(**kwargs)
        context["page_title"] = self.get_page_title()
        context["page_description"] = self.get_page_description()
        context["menu"] = menu(user=self.request.user, request=self.request)
        context["breadcrumbs"] = self.get_breadcrumbs()
        context["tabs"] = self.get_tabs()
        context["index_url"] = self.get_index_url()
        context["SITE_NAME"] = self.get_site_name()
        context["SITE_TITLE"] = self.get_site_title()
        context["SITE_LOGO"] = self.get_site_logo()
        context["SITE_FAVICON"] = self.get_site_favicon()
        context["SIDEBAR_BACKGROUND"] = self.get_sidebar_background()
        context["SIDEBAR_COLOR"] = self.get_sidebar_color()
        context["SIDEBAR_ALT_COLOR"] = self.get_sidebar_alt_color()
        context["HIGHLIGHT_BACKGROUND"] = self.get_highlight_background()
        context["SIDEBAR_ALT_COLOR_DARK"] = self.get_sidebar_alt_color_dark()
        context["SIDEBAR_COLOR_DARK"] = self.get_sidebar_color_dark()
        context["SIDEBAR_BACKGROUND_DARK"] = self.get_sidebar_background_dark()
        context["HIGHLIGHT_COLOR"] = self.get_highlight_color()
        context["DATETIME_FORMATS"] = self.get_datetime_formats()
        context["LOGIN_URL"] = self.get_login_url()
        context["LOGOUT_URL"] = self.get_logout_url()
        context["media"] = self.media
        context["form_display"] = self.get_form_display()
        context["in_modal"] = self.request.GET.get("inmodal", False)
        context["dark_mode"] = self.get_dark_mode()
        return context

    def get_breadcrumbs(self):
        """
        Breadcrumb format: (('name', 'url'), ...) or None if not used.
        """
        if not self.breadcrumbs:
            return None
        else:
            allowed_breadcrumbs = []
            for breadcrumb in self.breadcrumbs:

                # check permission based on named_url
                if breadcrumb[1] is not None and not view_from_url(
                    breadcrumb[1]
                ).has_permission(self.request.user):
                    continue

                obj = self if not hasattr(self, "object") else self.object
                url = (
                    None
                    if not breadcrumb[1]
                    else reverse_url(breadcrumb[1], obj)
                )
                allowed_breadcrumbs.append({"name": breadcrumb[0], "url": url})
            return allowed_breadcrumbs

    def get_tabs(self):
        """
        Tabs format: (('name', 'url'), ...) or None if tabs are not used.
        """
        if not self.tabs:
            return None
        else:
            allowed_tabs = []
            for tab in self.tabs:

                # check permission based on named_url
                if not view_from_url(tab[1]).has_permission(self.request.user):
                    continue

                obj = self if not hasattr(self, "object") else self.object
                url = reverse_url(tab[1], obj)
                allowed_tabs.append(
                    {
                        "name": tab[0],
                        "active": self.request.path == url,
                        "url": self.in_modal(url),
                    }
                )
            return allowed_tabs

    def get_page_title(self):
        return self.page_title

    def get_page_description(self):
        return self.page_description

    def get_site_logo(self):
        return arctic_setting("ARCTIC_SITE_LOGO")

    def get_site_favicon(self):
        return getattr(settings, "ARCTIC_SITE_FAVICON", None)

    def get_site_name(self):
        return arctic_setting("ARCTIC_SITE_NAME")

    def get_site_title(self):
        return getattr(settings, "ARCTIC_SITE_TITLE", self.get_site_name())

    def get_sidebar_color(self):
        return getattr(settings, "ARCTIC_SIDEBAR_COLOR", None)

    def get_sidebar_alt_color(self):
        return getattr(settings, "ARCTIC_SIDEBAR_ALT_COLOR", None)

    def get_sidebar_background(self):
        return getattr(settings, "ARCTIC_SIDEBAR_BACKGROUND", None)

    def get_highlight_color(self):
        return getattr(settings, "ARCTIC_HIGHLIGHT_COLOR", None)

    def get_highlight_background(self):
        return getattr(settings, "ARCTIC_HIGHLIGHT_BACKGROUND", None)

    def get_sidebar_alt_color_dark(self):
        return getattr(settings, "ARCTIC_SIDEBAR_ALT_COLOR_DARK", None)

    def get_sidebar_color_dark(self):
        return getattr(settings, "ARCTIC_SIDEBAR_COLOR_DARK", None)

    def get_sidebar_background_dark(self):
        return getattr(settings, "ARCTIC_SIDEBAR_BACKGROUND_DARK", None)

    def get_dark_mode(self):
        return getattr(settings, "ARCTIC_DARK_MODE", None)

    def get_index_url(self):
        try:
            return reverse(getattr(settings, "ARCTIC_INDEX_URL", "index"))
        except NoReverseMatch:
            return "/"

    def get_datetime_formats(self):
        dtformats = {}

        dtformats["SHORT_DATE"] = get_format(
            "DATE_INPUT_FORMATS", get_language()
        )[0]
        dtformats["TIME"] = get_format("TIME_INPUT_FORMATS", get_language())[0]
        dtformats["SHORT_DATETIME"] = get_format(
            "DATETIME_INPUT_FORMATS", get_language()
        )[0]
        firstweekday = calendar.firstweekday() + 1
        if firstweekday == 7:
            firstweekday = 0
        day_names = calendar.day_name[-1:] + calendar.day_name[:-1]
        day_abbr = calendar.day_abbr[-1:] + calendar.day_abbr[:-1]
        dtformats["DAYS"] = mark_safe(
            json.dumps([_(day) for day in day_names])
        )
        dtformats["DAYS_SHORT"] = mark_safe(
            json.dumps([_(day) for day in day_abbr])
        )
        dtformats["DAYS_MIN"] = mark_safe(
            json.dumps([_(day)[:2] for day in day_abbr])
        )
        dtformats["MONTHS"] = mark_safe(
            json.dumps(
                [_(day).capitalize() for day in calendar.month_name[1:]]
            )
        )
        dtformats["MONTHS_SHORT"] = mark_safe(
            json.dumps([_(day) for day in calendar.month_abbr[1:]])
        )
        dtformats["TODAY"] = _("Today")
        dtformats["CLEAR"] = _("Clear")
        dtformats["FIRST_DAY"] = firstweekday

        return dtformats

    def get_login_url(self):
        login_url = getattr(settings, "LOGIN_URL", "login")
        return reverse(login_url) if login_url else None

    def get_logout_url(self):
        logout_url = getattr(settings, "LOGOUT_URL", "logout")
        return reverse(logout_url) if logout_url else None

    @property
    def media(self):
        """
        Return all media required to render this view, including forms.
        """
        media = self._get_common_media()
        media += self._get_view_media()
        media += self.get_media_assets()
        return media

    def _get_common_media(self):
        config = getattr(settings, "ARCTIC_COMMON_MEDIA_ASSETS", {})
        css = config["css"] if "css" in config else {}
        js = config["js"] if "js" in config else []
        media = Media(css=css, js=js)
        return media

    def get_media_assets(self):
        """
        Allows to define additional media for view
        """
        return Media()

    def _get_view_media(self):
        """
        Gather view-level media assets
        """
        try:
            css = self.Media.css
        except AttributeError:
            css = {}
        try:
            js = self.Media.js
        except AttributeError:
            js = []
        return Media(css=css, js=js)

    def get_form_display(self):
        valid_options = ["stacked", "tabular", "float-label"]
        if self.form_display:
            if self.form_display in valid_options:
                return self.form_display
            raise ImproperlyConfigured(
                "form_display property needs to be one of {}".format(
                    valid_options
                )
            )
        return arctic_setting("ARCTIC_FORM_DISPLAY", valid_options)

    def in_modal(self, url):
        if self.request.GET.get("inmodal"):
            return append_query_parameter(url, {"inmodal": "True"})
        return url


class TemplateView(View, base.TemplateView):
    pass


class DetailView(View, base.DetailView):
    """
    Custom detail view.
    """

    fields = None  # Which fields should be shown in the details
    links = None  # Optional links such as list of linked items

    def get_fields(self, obj):
        result = OrderedDict()
        if self.fields:
            for field_name in self.fields:
                if isinstance(field_name, tuple):
                    # custom property that is not a field of the model
                    result[field_name[1]] = getattr(obj, field_name[0])
                else:
                    field = self.model._meta.get_field(field_name)
                    result[field.verbose_name.title()] = getattr(
                        obj, field_name
                    )
        return result

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        context["fields"] = self.get_fields(context["object"])
        context["links"] = self.get_links()
        return context


class ListView(View, ListMixin, base.ListView):
    """
    Custom listview. Adding filter, sorting and display logic.
    """

    prefix = ""  # Prefix for embedding multiple list views in detail view
    allowed_exports = []  # to display an export link

    def __init__(self, **kwargs):
        super(ListView, self).__init__(**kwargs)
        if self.sorting_field:
            resource = "{0}.{1}".format(
                self.__class__.__module__, self.__class__.__name__
            )
            try:
                self.sorting_url = reverse("arctic:order", args=(resource,))
            except NoReverseMatch:
                raise ImproperlyConfigured(
                    "This project needs to include Arctic URLs in order to use"
                    " sorting."
                )

            sorting_field_excludes = (
                self.ordering_fields,
                self.search_fields,
                self.default_ordering,
                self.advanced_search_form,
                self.paginate_by,
            )
            if any(sorting_field_excludes):
                raise ImproperlyConfigured(
                    "sorting_field cannot be used in combination with any of "
                    "the following properties: ordering_fields, search_fields,"
                    " default_ordering, advanced_search_form, paginate_by"
                )

    def get(self, request, *args, **kwargs):
        objects = self.get_object_list()
        context = self.get_context_data(object_list=objects)

        # checks if user want to export data to file
        response_format = request.GET.get("format", None)
        if response_format in self.allowed_exports:
            if response_format == "csv":
                return self.csv_file_response()

        return self.render_to_response(context)

    def get_export_url(self, format):
        return append_query_parameter(
            self.request.get_full_path(), {"format": format}
        )

    def get_object_list(self):
        qs = self.get_queryset()

        if self.advanced_search_form_class:
            form = self.get_advanced_search_form(data=self.request.GET)
            if form.is_valid():
                if not hasattr(form, "get_search_filter"):
                    raise AttributeError(
                        "advanced_search_form "
                        "must implement get_search_filter()"
                    )
                qs = qs.filter(form.get_search_filter())

        if self.get_simple_search_form_class():
            form = self.get_simple_search_form(self.request.GET)
            if form.is_valid():
                if not hasattr(form, "get_search_filter"):
                    raise AttributeError(
                        "simple_search_form "
                        "must implement get_search_filter()"
                    )
                qs = qs.filter(form.get_search_filter())

        self.object_list = qs
        return self.object_list

    def get_list_header(self):
        """
        Creates a list of dictionaries with the field names, labels,
        field links, field css classes, order_url and order_direction,
        this simplifies the creation of a table in a template.
        """
        model = self.object_list.model
        result = []
        if not self.get_fields():
            result.append(
                {"name": "", "verbose": str(model._meta.verbose_name)}
            )
        else:
            prefix = self.get_prefix()
            ordering_fields = self.get_ordering_fields()
            for field_name in self.get_fields():
                item = {}
                if isinstance(field_name, tuple):
                    # custom property that is not a field of the model
                    name = field_name[0]
                    item["label"] = field_name[1]
                else:
                    name = field_name
                    try:
                        field_meta = find_field_meta(model, field_name)
                        if field_meta._verbose_name:  # noqa
                            # explicitly set on the model, so don't change
                            item["label"] = field_meta._verbose_name  # noqa
                        else:
                            # title-case the field name (issue #80)
                            item["label"] = field_meta.verbose_name.title()
                    except FieldDoesNotExist:
                        item["label"] = field_name
                    except AttributeError:
                        item["label"] = field_name
                item["name"] = prefix + name
                if name in ordering_fields:
                    (
                        item["order_url"],
                        item["order_direction"],
                    ) = self.ordering_url(name)
                result.append(item)

        return result

    def get_list_items(self, objects):  # noqa: C901
        items = []
        if not self.get_fields():
            for obj in objects:
                items.append([obj.pk, str(obj)])
            return items

        # remove all tuples in the field list, no need for the verbose
        # field name here
        fields = self.get_fields(strip_labels=True)
        field_links = self.get_field_links()

        for obj in objects:
            field_classes = self.get_field_classes(obj)
            row = {
                "id": self.get_primary_value(obj),  # for row data id attr
                "fields": [],
                "actions": [],
                "sorting_field": None,
            }

            for field_name in fields:
                field = {"type": "field", "field": field_name}
                base_field_name = field_name.split("__")[0]
                field_class = get_field_class(objects, base_field_name)
                field["value"] = self.get_field_value(field_name, obj)
                if field_class == "ManyToManyField":
                    #  ManyToManyField will be display as an embedded list
                    #  capped to max_embeded_list_items, an ellipsis is
                    #  added if there are more items than the max.
                    m2mfield = getattr(obj, base_field_name)
                    embeded_list = list(
                        str(l)
                        for l in m2mfield.all()[
                            : self.max_embeded_list_items + 1
                        ]
                    )
                    if len(embeded_list) > self.max_embeded_list_items:
                        embeded_list = embeded_list[:-1] + ["..."]
                    field["value"] = embeded_list
                # don't try to find url for value that
                # is None for related objects.
                if (
                    field_name in field_links.keys()
                    and field["value"] is not None
                ):
                    field["url"] = self.in_modal(
                        reverse_url(
                            field_links[field_name], obj, self.primary_key
                        )
                    )
                    field["modal"] = self.get_modal_link(
                        field_links[field_name], obj
                    )
                if field_name in field_classes:
                    field["class"] = field_classes[field_name]
                row["fields"].append(field)

            actions = self._get_field_actions(obj)
            if actions:
                row["actions"].extend(actions)
            if self.sorting_field:
                row["sorting_field"] = {
                    "type": "sorting",
                    "id": self.get_primary_value(obj),
                    "value": getattr(obj, self.sorting_field),
                }
            items.append(row)
        return items

    def get_field_value(self, field_name, obj):
        # first try to find a virtual field
        virtual_field_name = "get_{}_field".format(field_name)
        if hasattr(self, virtual_field_name):
            return getattr(self, virtual_field_name)(obj)
        try:
            # Get the choice display value
            parent_objs = "__".join(field_name.split("__")[:-1])
            method_name = "{}__get_{}_display".format(
                parent_objs, field_name.split("__")[-1]
            ).strip("__")
            return find_attribute(obj, method_name)()
        except (AttributeError, TypeError):
            # finally get field's value
            return find_attribute(obj, field_name)

    def get_primary_value(self, obj):
        # while using annotate (group by) the object is a dict
        if type(obj) == dict:
            return obj.get(self.primary_key)

        return getattr(obj, self.primary_key)

    def get_prefix(self):
        return self.prefix + "-" if self.prefix else ""

    def get_default_ordering(self):
        prefix = self.get_prefix()
        return [prefix + f for f in self.default_ordering]

    def get_ordering_with_prefix(self):
        return self.request.GET.getlist("order", self.get_default_ordering())

    def get_ordering(self):
        """Ordering used for queryset filtering (should not contain prefix)."""
        if self.sorting_field:
            return [self.sorting_field]
        prefix = self.get_prefix()
        fields = self.get_ordering_with_prefix()
        if self.prefix:
            fields = [f.replace(prefix, "", 1) for f in fields]
        return [
            f
            for f in fields
            if f.lstrip("-") in self.get_ordering_fields_lookups()
        ]

    def get_page_title(self):
        if not self.page_title:
            return capfirst(self.object_list.model._meta.verbose_name_plural)
        return self.page_title

    def get_context_data(self, **kwargs):
        context = super(ListView, self).get_context_data(**kwargs)
        context["prefix"] = self.prefix
        context["list_header"] = self.get_list_header()
        context["list_items"] = self.get_list_items(context["object_list"])
        context["tool_links"] = self.get_tool_links()
        context["sorting_url"] = self.sorting_url
        context["tool_links_icon"] = self.get_tool_links_icon()
        context["tool_links_collapse"] = self.tool_links_collapse - int(
            len(context["tool_links"]) == self.tool_links_collapse + 1
        )
        context["simple_search_form"] = self.simple_search_form
        context["advanced_search_form"] = self.advanced_search_form
        return context

    def csv_file_response(self):
        """
        Create and return the HttpResponse object with the appropriate CSV data
        """
        model = self.model
        if model is None:
            model = self.queryset.model

        titles = []
        displayed_fields = []
        for field in self.get_fields():
            if isinstance(field, tuple):
                displayed_fields.append(field[0])
                titles.append(field[1].capitalize())
            else:
                displayed_fields.append(field)
                titles.append(field.capitalize())

        file_name = self.get_page_title()
        response = HttpResponse(content_type="text/csv")
        response["Content-Disposition"] = "attachment; filename={}.csv".format(
            file_name
        )

        # create and write the csv file
        writer = csv.writer(response)
        writer.writerow(titles)

        m2m_fields = [m2m_f.attname for m2m_f in model._meta.many_to_many]
        for obj in self.get_object_list():
            row = []
            for field in displayed_fields:
                try:
                    field_value = getattr(obj, field)
                except AttributeError:
                    field_value = self.get_field_value(field, obj)
                # checks related_field and get relevant values
                related_field = self._field_is_m2m(m2m_fields, field)
                if related_field:
                    related_manager = getattr(obj, related_field)
                    field_value = ", ".join(
                        [str(obj) for obj in related_manager.get_queryset()]
                    )
                # checks if still didn't get relevant 'field_value' value
                if isinstance(field_value, Manager):
                    field_value = ", ".join(
                        [str(obj) for obj in field_value.get_queryset()]
                    )
                row.append(strip_tags(field_value))
            writer.writerow(row)

        return response

    @classmethod
    def reorder(cls, rows):
        for key, value in rows.items():
            kv = {cls.primary_key: key, cls.sorting_field: value}
            cls.model(**kv).save(update_fields=[cls.sorting_field])


class DataListView(TemplateView, ListMixin):
    dataset = None
    template_name = "arctic/base_list.html"
    page_kwarg = "page"

    def get_ordering(self):
        return self.request.GET.get("order", "").split(",")

    def get_context_data(self, **kwargs):
        context = super(DataListView, self).get_context_data(**kwargs)
        page_size = self.get_paginate_by()
        self.dataset.page_size = page_size
        object_list = self.get_objects()
        page_context = {
            "paginator": None,
            "page_obj": None,
            "is_paginated": False,
            "object_list": object_list,
        }
        if page_size:
            paginator, page, object_list, is_paginated = self.paginate_dataset(
                object_list, page_size
            )
            page_context = {
                "paginator": paginator,
                "page_obj": page,
                "is_paginated": is_paginated,
                "object_list": object_list,
            }
        context.update(page_context)
        context["list_header"] = self.get_list_header()
        context["list_items"] = self.get_list_items(object_list)
        context["tool_links"] = self.get_tool_links()
        context["tool_links_icon"] = self.get_tool_links_icon()
        context["tool_links_collapse"] = self.tool_links_collapse - int(
            len(context["tool_links"]) == self.tool_links_collapse + 1
        )
        context["simple_search_form"] = self.simple_search_form
        context["advanced_search_form"] = self.advanced_search_form

        return context

    def get_paginate_by(self):
        return self.paginate_by

    def get_list_header(self):
        """
        Creates a list of dictionaries with the field names, labels,
        field links, field css classes, order_url and order_direction,
        this simplifies the creation of a table in a template.
        """
        result = []
        for field_name in self.get_fields():
            item = {}
            if isinstance(field_name, tuple):
                # custom property that is not a field of the model
                item["name"] = field_name[0]
                item["label"] = field_name[1]
            else:
                item["name"] = field_name
                item["label"] = field_name.title()
            if item["name"] in self.get_ordering_fields():
                item["order_url"], item["order_direction"] = self.ordering_url(
                    item["name"]
                )
            result.append(item)

        return result

    def get_objects(self):
        objects = self.dataset.fields(
            self.get_fields(strip_labels=False)
        ).order_by(self.get_ordering())
        search_fields = self.get_search_fields()
        if search_fields:
            if len(search_fields) > 1:
                raise ValidationError(
                    "search_fields in DataListView currently"
                    " only accepts one field"
                )
            search = {}
            for search_field in search_fields:
                value = self.request.GET.get("search", "")
                search[search_field] = value
            if search:
                objects = objects.filter(**search)
        return objects

    def get_list_items(self, objects):
        items = []
        fields = []
        field_links = self.get_field_links()
        for field in self.get_fields():
            fields.append(field[0] if type(field) in (list, tuple) else field)
        for obj in objects:
            field_classes = self.get_field_classes(obj)
            row = {"id": getattr(obj, "id", ""), "fields": [], "actions": []}
            for field_name in fields:
                field = {"field": field_name, "type": "field"}
                field["value"] = self.get_field_value(field_name, obj)
                if field_name in field_links.keys():
                    field["url"] = reverse_url(
                        field_links[field_name], obj, self.primary_key
                    )
                if field_name in field_classes:
                    field["class"] = field_classes[field_name]
                row["fields"].append(field)
            items.append(row)
            actions = self._get_field_actions(obj)
            if actions:
                row["actions"].append(actions)
        return items

    def get_field_value(self, field_name, obj):
        try:  # first try to find a virtual field
            virtual_field_name = "get_{}_field".format(field_name)
            return getattr(self, virtual_field_name)(obj)
        except AttributeError:
            try:
                return obj[field_name]
            except KeyError:
                raise FieldDoesNotExist(
                    'Field "{}" is not available'.format(field_name)
                )

    def get_paginator(
        self,
        dataset,
        per_page,
        orphans=0,
        allow_empty_first_page=True,
        **kwargs
    ):
        """Return an instance of the paginator for this view."""
        return IndefinitePaginator(
            dataset,
            per_page,
            orphans=orphans,
            allow_empty_first_page=allow_empty_first_page,
            **kwargs
        )

    def paginate_dataset(self, dataset, page_size):
        paginator = self.get_paginator(
            dataset, page_size, orphans=0, allow_empty_first_page=True
        )
        page_kwarg = self.page_kwarg
        page = (
            self.kwargs.get(page_kwarg)
            or self.request.GET.get(page_kwarg)
            or 1
        )
        try:
            page_number = int(page)
        except ValueError:
            raise Http404(_("Page cannot be converted to an int."))
        try:
            page = paginator.page(page_number)
            return (paginator, page, page.object_list, page.has_other_pages())
        except InvalidPage as e:
            raise Http404(
                _("Invalid page (%(page_number)s): %(message)s")
                % {"page_number": page_number, "message": str(e)}
            )


class CreateView(
    FormMediaMixin,
    View,
    SuccessMessageMixin,
    FormMixin,
    extra_views.CreateWithInlinesView,
):
    template_name = "arctic/base_create_update.html"
    success_message = _('"%(object)s" was successfully created')

    def get_page_title(self):
        if not self.page_title:
            return _("Create %s") % self.model._meta.verbose_name
        return self.page_title

    def get_context_data(self, **kwargs):
        context = super(CreateView, self).get_context_data(**kwargs)
        context["actions"] = self.get_actions()
        context["layout"] = self.get_layout()
        context["cancel_url"] = self.get_cancel_url()
        return context


class UpdateView(
    FormMediaMixin,
    SuccessMessageMixin,
    FormMixin,
    View,
    extra_views.UpdateWithInlinesView,
):
    template_name = "arctic/base_create_update.html"
    success_message = _('"%(object)s" was successfully updated')

    links = None  # Optional links such as list of linked items
    readonly_fields = None  # Optional list of readonly fields

    def get_page_title(self):
        if not self.page_title:
            return _("Edit %s") % self.model._meta.verbose_name
        return self.page_title

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        context["actions"] = self.get_actions()
        context["layout"] = self.get_layout()
        context["cancel_url"] = self.get_cancel_url()
        return context


class FormView(
    FormMediaMixin, View, SuccessMessageMixin, FormMixin, base.FormView
):
    template_name = "arctic/base_create_update.html"

    def get_context_data(self, **kwargs):
        context = super(FormView, self).get_context_data(**kwargs)
        context["actions"] = self.get_actions()
        context["layout"] = self.get_layout()
        context["cancel_url"] = self.get_cancel_url()
        return context


class DeleteView(View, base.DeleteView):
    template_name = "arctic/base_confirm_delete.html"
    redirect = True

    @staticmethod
    def confirm_dialog():
        return {
            "title": _('Delete "{obj}"'),
            "message": _("Are you sure you want to delete this?"),
            "ok": _("Delete"),
            "cancel": _("Cancel"),
        }

    def get_success_url(self):
        if self.success_url:
            return self.success_url
        if self.request.GET.get("inmodal"):
            return reverse("arctic:redirect_to_parent")
        return self.request.META.get("HTTP_REFERER")

    def get_success_message(self, obj):
        return _('"{}" was successfully deleted.').format(str(obj))

    def get(self, request, *args, **kwargs):
        """
        Catch protected relations and show to user.
        """
        self.object = self.get_object()
        can_delete = True
        protected_objects = []
        collector_message = None
        collector = Collector(using="default")
        try:
            collector.collect([self.object])
        except ProtectedError as e:
            collector_message = (
                "Cannot delete %s because it has relations "
                "that depends on it." % self.object
            )
            protected_objects = e.protected_objects
            can_delete = False

        if can_delete and self.redirect:
            messages.success(request, self.get_success_message(self.object))
            return self.delete(request, *args, **kwargs)

        context = self.get_context_data(
            object=self.object,
            can_delete=can_delete,
            collector_message=collector_message,
            protected_objects=protected_objects,
        )
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        response = super(DeleteView, self).post(request, *args, **kwargs)
        messages.success(request, self.get_success_message(self.object))
        return response


class LoginView(SuccessURLAllowedHostsMixin, FormView):
    template_name = "arctic/login.html"
    page_title = _("Login")
    requires_login = False
    redirect_authenticated_user = False
    form_class = AuthenticationForm

    def __init__(self, *args, **kwargs):
        super(LoginView, self).__init__(*args, **kwargs)
        # thread-safe definition of messages.
        self.messages = []

    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super(LoginView, self).get_context_data(**kwargs)
        context["next"] = self.request.GET.get("next", "/")
        context["username"] = self.request.POST.get("username", "")
        context["messages"] = set(self.messages)
        return context

    def get_success_url(self):
        url = self.get_redirect_url()
        return url or resolve_url(settings.LOGIN_REDIRECT_URL)

    def get_redirect_url(self):
        """Return the user-originating redirect URL if it's safe."""
        redirect_to = self.request.POST.get(
            self.redirect_field_name,
            self.request.GET.get(self.redirect_field_name, ""),
        )
        url_is_safe = url_has_allowed_host_and_scheme(
            url=redirect_to,
            allowed_hosts=self.get_success_url_allowed_hosts(),
            require_https=self.request.is_secure(),
        )
        return redirect_to if url_is_safe else ""

    def get(self, request, *args, **kwargs):
        # If the logout url is the login url, log the user out of the system
        if settings.LOGOUT_URL == settings.LOGIN_URL:
            logout(request)
        # Else redirect a logged in user to the homepage
        elif (
            self.redirect_authenticated_user
            and self.request.user.is_authenticated
        ):
            redirect_to = self.get_success_url()
            if redirect_to == self.request.path:
                raise ValueError(
                    "Redirection loop for authenticated user detected. Check that "
                    "your LOGIN_REDIRECT_URL doesn't point to a login page."
                )
            return HttpResponseRedirect(redirect_to)
        return super(LoginView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        user = authenticate(
            username=request.POST["username"],
            password=request.POST["password"],
        )
        if user and user.is_active:
            login(request, user)

            next_url = request.GET.get("next")
            if url_has_allowed_host_and_scheme(next_url, request.get_host()):
                return redirect(next_url)

            return redirect("/")

        self.messages.append(_("Invalid username/password combination"))

        return render(
            request, self.template_name, self.get_context_data(**kwargs)
        )


class LogoutView(View):
    def dispatch(self, request, *args, **kwargs):
        logout(request)
        return redirect(settings.LOGIN_URL)



import pytest

from django.urls import reverse

from articles.views import ArticleListView
from tests.factories import ArticleFactory


@pytest.mark.django_db
class TestListView(object):
    """
    Test ListView using ArticleListView from example application
    """
    def _request(self, admin_client):
        response = admin_client.get(reverse('articles:list'))
        assert response.status_code == 200
        return response

    def _assert_list_items_len(self, response, length):
        assert 'list_items' in response.context_data
        assert len(response.context_data['list_items']) == length

    def test_single_item(self, admin_client):
        """
        Single item in ListView
        """
        article = ArticleFactory()
        response = self._request(admin_client)
        self._assert_list_items_len(response, 1)
        lst_items = response.context_data['list_items']
        field = lst_items[0]['fields'][0]['field']
        assert lst_items[0]['fields'][0]['value'] == getattr(article, field)

    def test_paginated_items(self, admin_client):
        """
        Paginated List view
        """
        [ArticleFactory() for i in range(30)]

        response = self._request(admin_client)
        self._assert_list_items_len(response, 10)

        assert response.context_data['page_obj'].paginator.num_pages == 3

    def test_no_items(self, admin_client):
        """
        No items in ListView
        """
        response = self._request(admin_client)

        self._assert_list_items_len(response, 0)

    def test_two_tool_links(self, admin_client):
        """
        Single tool link display
        """
        response = self._request(admin_client)

        assert 'tool_links' in response.context_data
        assert len(response.context_data['tool_links']) == 2

    def test_multiple_tool_links(self, admin_client):
        """
        Multiple tool links display
        """
        ArticleListView.tool_links.append(('Create', 'articles:create'))

        response = self._request(admin_client)

        assert 'tool_links' in response.context_data
        assert len(response.context_data['tool_links']) == 3

    def test_no_tool_links(self, admin_client):
        """
        Handling case with no tool links
        """
        ArticleListView.tool_links = []

        response = self._request(admin_client)

        assert 'tool_links' in response.context_data
        assert len(response.context_data['tool_links']) == 0

    def test_simple_search_form(self, admin_client):
        """
        Test simple search form
        """
        ArticleFactory(title='title1')
        ArticleFactory(title='title2')

        # empty search, return all
        response = admin_client.get(reverse('articles:list'), {'search': ''})
        assert len(response.context_data['list_items']) == 2

        # search filled, filter content
        response = admin_client.get(reverse('articles:list'),
                                    {'search': 'le1'})

        list_items = response.context_data['list_items']
        assert len(list_items) == 1
        assert list_items[0]['fields'][0]['value'] == 'title1'

    def test_advanced_search_form(self, admin_client):
        """
        Test simple search form
        """
        ArticleFactory(title='title1', description='description1')
        ArticleFactory(title='title2', description='description2')

        # empty search, return all
        response = admin_client.get(reverse('articles:list'),
                                    {'description': ''})
        assert len(response.context_data['list_items']) == 2

        # search filled, filter content
        response = admin_client.get(reverse('articles:list'),
                                    {'description': 'tion2'})
        list_items = response.context_data['list_items']
        assert len(list_items) == 1
        assert list_items[0]['fields'][0]['value'] == 'title2'

    def test_simple_search_form_quick_filters(self, admin_client):
        """
        Test quick filters result
        """
        ArticleFactory(
            title='title1',
            description='description1',
            published=True
        )
        ArticleFactory(
            title='title2',
            description='Dead rabbit is walking without leg.',
            published=False
        )

        response = admin_client.get(reverse('articles:list'),
                                    {'published': 'published'})
        list_items = response.context_data['list_items']
        assert len(list_items) == 1
        assert list_items[0]['fields'][0]['value'] == 'title1'

        response = admin_client.get(reverse('articles:list'),
                                    {'published': 'drafts'})
        list_items = response.context_data['list_items']
        assert list_items[0]['fields'][0]['value'] == 'title2'

    def test_field_actions(self, admin_client):
        ArticleFactory(title='title1', description='description1',
                       published=True)
        ArticleFactory(title='title2', description='description2',
                       published=False)

        response = admin_client.get(reverse('articles:list'),
                                    {'description': ''})
        list_items = response.context_data['list_items']
        assert len(list_items[0]['actions']) == 1
        assert len(list_items[1]['actions']) == 2

    def test_field_actions_allowing(self, admin_client):
        def get_field_actions(self, obj):
            action_links = [
                ('detail', 'articles:detail', 'fa-edit'),
                ('categories', 'articles:category-list', 'fa-edit'),
            ]
            if not obj.published:
                action_links.append(
                    ('delete', 'articles:delete', 'fa-trash'), )
            return action_links

        ArticleListView.get_field_actions = get_field_actions

        ArticleFactory(title='title1', description='description1',
                       published=True)
        ArticleFactory(title='title2', description='description2',
                       published=False)

        response = admin_client.get(reverse('articles:list'),
                                    {'description': ''})
        list_items = response.context_data['list_items']
        assert len(list_items[0]['actions']) == 1
        assert len(list_items[1]['actions']) == 2

    def test_export_csv_file(self, admin_client):
        """
        Test csv file export functional
        """
        for i in range(30):
            ArticleFactory(title="title{}".format(i), description='description{}'.format(i))

        response = admin_client.get(reverse('articles:list'), {'format': 'csv'})
        assert response.status_code == 200
        assert response._headers['content-type'][1] == 'text/csv'
        attach_file = 'Articles.csv'
        assert attach_file in response._headers['content-disposition'][1]
        # 2 more rows in file: titles and blank line
        assert len(response._container) == 32



from collections import OrderedDict

from django.http import HttpRequest
from django.urls import ResolverMatch, reverse
from django.utils.translation import gettext_lazy as _

import pytest
from arctic import utils


@pytest.mark.django_db
class TestMenu:
    OUTPUT_MENU = OrderedDict([
        (_('Dashboard'), {
            'submenu': None, 'active': False,
            'active_weight': 1, 'icon': 'fa-dashboard', 'url': 'index'}
         ),
        (_('Articles'), {
            'submenu': OrderedDict([
                (_('List'), {'submenu': None,
                             'active': False,
                             'active_weight': 10, 'icon': None,
                             'url': 'articles:list'}),
                (_('Create'), {'submenu': None, 'active': False,
                               'active_weight': 17, 'icon': None,
                               'url': 'articles:create'})
            ]),
            'active': False,
            'active_weight': 0, 'icon': 'fa-file-text-o', 'url': None}
         ),
        (_('Categories'), {
            'submenu': OrderedDict([
                (_('List'), {'submenu': None, 'active': False,
                             'active_weight': 19, 'icon': None,
                             'url': 'articles:category-list'}
                 ),
                (_('Create'), {
                    'submenu': None, 'active': False,
                    'active_weight': 26, 'icon': None,
                    'url': 'articles:category-create'})
            ]),
            'active': False, 'active_weight': 0,
            'icon': 'fa-sitemap', 'url': None}
         ),
        (_('Tags'), {
            'submenu': OrderedDict([
                (_('List'), {'submenu': None, 'active': False,
                             'active_weight': 15, 'icon': None,
                             'url': 'articles:tag-list'}),
                (_('Create'), {'submenu': None, 'active': False,
                               'active_weight': 22, 'icon': None,
                               'url': 'articles:tag-create'})
            ]),
            'active': False, 'active_weight': 0,
            'icon': 'fa-tags', 'url': None}
         ),
        (_('Users'), {
            'submenu': OrderedDict([
                (_('List'), {'submenu': None, 'active': False,
                             'active_weight': 7, 'icon': None,
                             'url': 'users:list'}),
                (_('Create'), {'submenu': None, 'active': False,
                               'active_weight': 14, 'icon': None,
                               'url': 'users:create'})
            ]),
            'active': False, 'active_weight': 0,
            'icon': 'fa-user', 'url': None}
         ),
        (_('Countries'), {'submenu': None, 'active': False,
                          'active_weight': 11, 'icon': 'fa-globe',
                          'url': 'countries-list'})
    ])

    def test_menu(self, admin_user):
        """
        Test menu items with and without icons and with and without submenus
        """
        path = '/'
        request = HttpRequest()
        request.path = path
        request.resolver_match = ResolverMatch(None, None, None,
                                               url_name='index')
        request.user = admin_user
        kwargs = {'request': request, 'user': admin_user}

        menu = utils.menu(**kwargs)
        assert menu == self.OUTPUT_MENU

    def test_menu_on_detail_page(self, admin_user):
        path = reverse('articles:detail', kwargs={'pk': 1})
        request = HttpRequest()
        request.path = path
        request.resolver_match = ResolverMatch(None, None, None,
                                               url_name='detail',
                                               app_names=['articles'],
                                               namespaces=['articles'])
        request.user = admin_user
        kwargs = {'request': request, 'user': admin_user}

        menu = utils.menu(**kwargs)
        assert dict(menu)['Articles']['active'] is True

        articles_submenu = dict(dict(menu)['Articles']['submenu'])
        assert articles_submenu['List']['active'] is True

    def test_is_active_menu_item(self):
        MENU_CONF = (
            ('Articles', None, 'fa-file-text-o', (
                ('List', 'articles:list', ('articles:detail',)),
                ('Create', 'articles:create')
            ), ('articles:delete',)),
        )
        is_menu_active = utils.is_active(MENU_CONF[0], 'articles:delete')
        is_submenu_active = utils.is_active(MENU_CONF[0][-2][0],
                                            'articles:delete')
        assert is_menu_active is True
        assert is_submenu_active is False



import pytest
from django.urls import reverse

from tests.factories import ArticleFactory


@pytest.mark.django_db
class TestFieldClasses(object):
    def _request(self, admin_client):
        response = admin_client.get(reverse('articles:list'))
        assert response.status_code == 200
        return response

    def test_field_classes(self, admin_client):
        """
        Virtual field displayed in ListView
        """
        ArticleFactory(published=True)
        ArticleFactory(published=False)
        response = self._request(admin_client)

        list_items = response.context_data['list_items']
        assert list_items[0]['fields'][2]['class'] == 'online'
        assert list_items[1]['fields'][2]['class'] == 'offline'



from django.urls import reverse


def test_article_list(admin_client, article):
    response = admin_client.get(reverse('articles:list'))
    assert response.status_code == 200
    list_items = response.context['list_items'][0]['fields']
    item_dict = {}
    for item in list_items:
        try:
            field = item['field']
            value = item['value']
        except (KeyError, TypeError):
            # Not a dict, so continue
            continue

        item_dict[field] = value

    assert item_dict['tags'][0] == 'Tag 0'
    assert item_dict['category'] == 'Category 0'
    assert item_dict['title'] == 'Article 0'
    assert item_dict['description'] == ''



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _


class UserRole(models.Model):
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        related_name="urole",
        on_delete=models.CASCADE,
    )
    role = models.ForeignKey("arctic.Role", on_delete=models.CASCADE)

    class Meta:
        swappable = "ARCTIC_USER_ROLE_MODEL"
        ordering = ["user"]


class Role(models.Model):
    name = models.CharField(_("Role"), max_length=100, unique=True)
    is_active = models.BooleanField(default=True)

    def __str__(self):
        return self.name

    class Meta:
        swappable = "ARCTIC_ROLE_MODEL"
        ordering = ["name"]



import pytest
from django.core.exceptions import PermissionDenied
from django.test import Client


class TestCMS(object):
    """
    Tests that probe the CMS with the Django test client
    """

    @pytest.fixture(scope="session", autouse=True)
    def init(self):
        self.__class__.client = Client()

    def test_login_success_redirect_no_next(self, admin_user):
        """
        Asserts that an admin user is redirected to root after login,
        when no 'next' query param is given.

        :param admin_user: fixture provided by pytest-django
        :return: void
        """
        response = self.client.post('/login/',
                                    {
                                        'username': 'admin',
                                        'password': 'password'
                                    })

        assert response.status_code == 302
        assert response.url in ('/', 'http://testserver/')

    def test_login_success_redirect_next(self, admin_user):
        """
        Asserts that an admin user is redirected to the next page after
        login, when a 'next' query param is given.

        :param admin_user: fixture provided by pytest-django
        :return: void
        """
        response = self.client.post('/login/?next=/articles/',
                                    {
                                        'username': 'admin',
                                        'password': 'password'
                                    })

        assert response.status_code == 302
        assert response.url in ('/articles/', 'http://testserver/articles/')

    def test_login_success_redirect_next_other_host(self, admin_user):
        """
        Asserts that an admin user is redirected to root after login,
        when the 'next' query param refers to another host.

        :param admin_user: fixture provided by pytest-django
        :return: void
        """
        response = self.client.post('/login/?next=http://www.no.domain/test/',
                                    {
                                        'username': 'admin',
                                        'password': 'password'
                                    })

        assert response.status_code == 302
        assert response.url in ('/', 'http://testserver/')

    def test_login_success(self, admin_user):
        """
        Asserts that an admin user can log in an see a restricted page
        within the CMS

        :param admin_user: fixture provided by pytest-django
        :return: void
        """
        self.client.post('/login/',
                         {
                             'username': 'admin',
                             'password': 'password'
                         })
        response = self.client.get('/articles/')

        assert response.status_code == 200

    def test_login_fail(self, admin_user):
        """
        Asserts that an unauthorized user cannot see a restricted page

        :return:
        """
        try:
            response = self.client.get('/articles/')
            assert response.status_code != 200
        except PermissionDenied:
            assert True



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-11-21 11:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('articles', '0002_auto_20171113_1527'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='order',
            field=models.IntegerField(blank=True, null=True, verbose_name='Order'),
        ),
    ]



# -*-*- encoding: utf-8 -*-*-
from __future__ import absolute_import, unicode_literals

from arctic.generics import TemplateView


class DashboardView(TemplateView):
    template_name = "arctic/index.html"
    page_title = "Dashboard"
    requires_login = False

    def get_context_data(self, **kwargs):
        context = super(DashboardView, self).get_context_data(**kwargs)
        return context



from django.conf.urls import include, url

from arctic.generics import LoginView, LogoutView
from arctic.views import handler400, handler403, handler404  # noqa


urlpatterns = [
    url(r"^login/$", LoginView.as_view(), name="login"),
    url(r"^logout/$", LogoutView.as_view(), name="logout"),
    url(r"^arctic/", include("arctic.urls", namespace="arctic")),
    url(r"^", include("dashboard.urls")),
]



"""
Basic mixins for generic class based views.
"""
import importlib
import sys
import warnings
from collections import OrderedDict

from django.conf import settings
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured, PermissionDenied
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from .forms import SimpleSearchForm
from .loading import get_role_model, get_user_role_model
from .utils import arctic_setting, generate_id, reverse_url, view_from_url
from .widgets import SelectizeAutoComplete

Role = get_role_model()
UserRole = get_user_role_model()


class SuccessMessageMixin(object):
    """
    Adds a success message on successful form submission.
    Altered to work with extra_views
    """

    success_message = ""
    message_delivered = False

    def form_valid(self, form):
        response = super(SuccessMessageMixin, self).form_valid(form)
        success_message = self.get_success_message(form.cleaned_data)
        if success_message and not self.message_delivered:
            messages.success(self.request, success_message)
            self.message_delivered = True
        return response

    def forms_valid(self, form, inlines):
        response = super(SuccessMessageMixin, self).forms_valid(form, inlines)
        success_message = self.get_success_message(form.cleaned_data)
        if success_message and not self.message_delivered:
            messages.success(self.request, success_message)
            self.message_delivered = True
        return response

    def get_success_message(self, cleaned_data):
        return self.success_message % dict(cleaned_data, object=self.object)


class ModalMixin(object):
    modal_links = {}

    def get_modal_link(self, url, obj={}):
        """
        Returns the metadata for a link that needs to be confirmed, if it
        exists, it also parses the message and title of the url to include
        row field data if needed.
        """
        if not (url in self.modal_links.keys()):
            return None

        try:
            if type(obj) != dict:
                obj.obj = str(obj)
                obj = vars(obj)
            link = self.modal_links[url]
            if link["type"] == "confirm":
                link["message"] = link["message"].format(**obj)
                link["title"] = link["title"].format(**obj)
                link["ok"]  # triggers a KeyError exception if not existent
                link["cancel"]
            elif link["type"] == "iframe":
                try:
                    link["size"]
                except KeyError:
                    link["size"] = "medium"
            else:
                raise ImproperlyConfigured(
                    "modal_links type: " + link["type"] + " is unsupported"
                )
            return link
        except KeyError as e:
            raise ImproperlyConfigured(
                "modal_links misses the following attribute: " + str(e)
            )
        except AttributeError:
            return None

    def _extract_confirm_dialog(self, view, url):
        dialog = getattr(view, "confirm_dialog", None)
        if dialog:
            self.modal_links[url] = dialog()
            self.modal_links[url]["type"] = "confirm"


class FormMixin(ModalMixin):
    """
    Adding customizable fields to view. Using the 12-grid system, you
    can now give fields a css-attribute. See reference for more information

    This is how layouts are built:
        Fieldset
          |-->  Rows
                  |--> Fields
    """

    use_widget_overloads = True
    layout = None
    actions = None  # Optional links such as list of linked items
    links = None
    readonly_fields = None
    ALLOWED_COLUMNS = 12  # There are 12 columns available
    form = None

    def get_cancel_url(self):
        if self.request.GET.get("inmodal"):
            return "javascript:modal_close()"

        return self.request.POST.get(
            "cancel_url",
            self.request.META.get(
                "HTTP_REFERER",
                "/".join(
                    self.request.get_full_path().rstrip("/").split("/")[:-1]
                ),
            ),
        )

    def get_success_url(self):
        """Return the URL to redirect to after processing a valid form."""
        if not self.success_url:
            if self.request.GET.get("inmodal"):
                return reverse("arctic:redirect_to_parent")
            raise ImproperlyConfigured(
                "No URL to redirect to. Provide a success_url."
            )
        return self.in_modal(str(self.success_url))  # success_url may be lazy

    def get_actions(self):  # noqa: C901
        if self.actions and self.links:
            raise ImproperlyConfigured(
                'Forms cannot have both "actions" and "links", please use '
                'only "actions"'
            )
        if self.links:
            warnings.warn(
                '"links" property is deprecated, please use "actions" '
                "instead.",
                DeprecationWarning,
            )
            self.actions = self.links

        # Forms require a submit
        default_action = {
            "label": _("Submit"),
            "type": "submit",
            "id": "action-submit",
            "style": "primary",
        }
        if not self.actions:
            return [default_action]

        allowed_actions = []
        last_submit_index = -1
        try:
            obj = self.get_object()
        except AttributeError:
            obj = None

        for action in self.actions:
            # check permission based on named_url
            if (action[1] in ("cancel", "submit")) or view_from_url(
                action[1]
            ).has_permission(self.request.user):
                allowed_action = {
                    "label": action[0],
                    "style": "secondary",
                    "id": generate_id("action", action[0]),
                }

                if action[1] == "submit":
                    allowed_action["type"] = action[1]
                elif action[1] == "cancel":
                    allowed_action["type"] = "link"
                    allowed_action["url"] = self.get_cancel_url()
                else:
                    self._extract_confirm_dialog(
                        view_from_url(action[1]), action[1]
                    )
                    allowed_action["modal"] = self.get_modal_link(
                        action[1], obj
                    )
                    allowed_action["type"] = "link"
                    allowed_action["url"] = self.in_modal(
                        reverse_url(action[1], obj)
                    )
                if len(action) == 3:
                    if action[2] == "left":
                        allowed_action["position"] = "left"
                    elif type(action[2]) is dict:
                        allowed_action.update(action[2])
                        if action[2].get("style"):
                            allowed_action["custom_style"] = True
                        if action[2].get("form_action"):
                            allowed_action["form_action"] = self.in_modal(
                                reverse_url(action[2]["form_action"], obj)
                            )

                if action[1] == "submit":
                    last_submit_index = len(allowed_actions)
                    allowed_action["name"] = generate_id(action[0])
                allowed_actions.append(allowed_action)

        if last_submit_index >= 0:
            if not allowed_actions[last_submit_index].get("custom_style"):
                allowed_actions[last_submit_index]["style"] = "primary"
        else:
            allowed_actions.append(default_action)
        return allowed_actions

    def get_layout(self, inline_layout=None, fields=None):
        layout = inline_layout or self.layout
        if not layout:
            return None

        if not inline_layout:
            fields = [field for field in self.get_form().fields]
        if not fields:
            return None

        allowed_rows = OrderedDict()
        py36_version = 0x30600F0  # hex number that represents python 3.6.0
        if (type(layout) is OrderedDict) or (
            (type(layout) is dict) and sys.hexversion >= py36_version
        ):
            for i, (fieldset, rows) in enumerate(layout.items()):
                fieldset = self._return_fieldset(fieldset)
                if isinstance(rows, str):
                    allowed_rows.update(
                        {i: {"fieldset": fieldset, "rows": rows}}
                    )
                else:
                    row = self._process_first_level(rows, fields)
                    allowed_rows.update(
                        {i: {"fieldset": fieldset, "rows": row}}
                    )

        elif type(layout) in (list, tuple):
            row = self._process_first_level(layout, fields)
            fieldset = self._return_fieldset(0)
            allowed_rows.update({0: {"fieldset": fieldset, "rows": row}})

        else:
            raise ImproperlyConfigured(
                "`layout` should be a list, tuple or "
                "a dict (OrderedDict if python < 3.6)"
            )
        return allowed_rows

    def _process_first_level(self, rows, fields):
        allowed_rows = []
        for row in rows:
            if isinstance(row, str):
                allowed_rows.append(self._return_field(row, fields))
            elif type(row) in (list, tuple):
                rows = self._process_row(row, fields)
                allowed_rows.append(rows)
        return allowed_rows

    def _process_row(self, row, fields):
        has_column = {}
        has_no_column = {}
        sum_existing_column = 0

        _row = {}
        for index, field in enumerate(row):
            # Yeah, like this isn't incomprehensible yet. Let's add recursion
            if type(field) in (list, OrderedDict):
                _row[index] = self._process_row(field)
            elif isinstance(field, str):
                name, column = self._split_str(field)
                if column:
                    has_column[index] = self._return_field(field, fields)
                    sum_existing_column += int(column)
                else:
                    has_no_column[index] = field

        col_avg, col_last = self._calc_avg_and_last_val(
            has_no_column, sum_existing_column
        )
        has_no_column = self._set_has_no_columns(
            has_no_column, col_avg, col_last, fields
        )

        # Merge it all back together to a dict, to preserve the order
        for index, field in enumerate(row):
            if index in has_column:
                _row[index] = has_column[index]
            if index in has_no_column:
                _row[index] = has_no_column[index]

        rows_to_list = [field for index, field in _row.items()]

        return rows_to_list

    def _set_has_no_columns(self, has_no_column, col_avg, col_last, fields):
        """
        Regenerate has_no_column by adding the amount of columns at the end
        """
        for index, field in has_no_column.items():
            if index == len(has_no_column):
                field_name = "{field}|{col_last}".format(
                    field=field, col_last=col_last
                )
                has_no_column[index] = self._return_field(field_name, fields)
            else:
                field_name = "{field}|{col_avg}".format(
                    field=field, col_avg=col_avg
                )
                has_no_column[index] = self._return_field(field_name, fields)
        return has_no_column

    def _return_fieldset(self, fieldset):
        """
        This function became a bit messy, since it needs to deal with two
        cases.

        1) No fieldset, which is represented as an integer
        2) A fieldset
        """
        collapsible = None
        description = None
        try:
            # Make sure strings with numbers work as well, do this
            int(str(fieldset))
            title = None
        except ValueError:
            if fieldset.count("|") > 1:
                raise ImproperlyConfigured(
                    "The fieldset name does not "
                    "support more than one | sign. "
                    "It's meant to separate a "
                    "fieldset from its description."
                )

            title = fieldset
            if "|" in fieldset:
                title, description = fieldset.split("|")
            if fieldset and (fieldset[0] in "-+"):
                if fieldset[0] == "-":
                    collapsible = "closed"
                else:
                    collapsible = "open"
                title = title[1:]

        return {
            "title": title,
            "description": description,
            "collapsible": collapsible,
        }

    def _calc_avg_and_last_val(self, has_no_column, sum_existing_columns):
        """
        Calculate the average of all columns and return a rounded down number.
        Store the remainder and add it to the last row. Could be implemented
        better. If the enduser wants more control, he can also just add the
        amount of columns. Will work fine with small number (<4) of items in a
        row.

        :param has_no_column:
        :param sum_existing_columns:
        :return: average, columns_for_last_element
        """
        sum_no_columns = len(has_no_column)
        columns_left = self.ALLOWED_COLUMNS - sum_existing_columns

        if sum_no_columns == 0:
            columns_avg = columns_left
        else:
            columns_avg = int(columns_left / sum_no_columns)

        remainder = columns_left - (columns_avg * sum_no_columns)
        columns_for_last_element = columns_avg + remainder
        return columns_avg, columns_for_last_element

    def _split_str(self, field):
        """
        Split title|7 into (title, 7)
        """
        field_items = field.split("|")
        if len(field_items) == 2:
            return field_items[0], field_items[1]
        elif len(field_items) == 1:
            return field_items[0], None

    def _return_field(self, field, fields):
        field_name, field_class = self._split_str(field)
        if field_name in fields:
            return {
                "name": field_name,
                "column": field_class,
                "field": self.get_form()[field_name],
            }
        else:
            return None

    def update_form_fields(self, form):
        widget_overloads = arctic_setting("ARCTIC_WIDGET_OVERLOADS")
        widgets_to_be_overloaded = widget_overloads.keys()
        if hasattr(form, "form_classes"):
            form_names = form.form_classes.keys()
            for form_name in form_names:
                new_form = self._update_form_fields(
                    form[form_name], widget_overloads, widgets_to_be_overloaded
                )
                setattr(form, form_name, new_form)
            return form

        return self._update_form_fields(
            form, widget_overloads, widgets_to_be_overloaded
        )

    def _update_form_fields(
        self, form, widget_overloads, widgets_to_be_overloaded
    ):
        for field in form.fields:
            if self.use_widget_overloads:
                widget_class = form.fields[field].widget.__class__.__name__
                if widget_class in widgets_to_be_overloaded:
                    module, wdgt = widget_overloads[widget_class].rsplit(
                        ".", 1
                    )
                    new_widget_module = importlib.import_module(module)
                    new_widget_class = getattr(new_widget_module, wdgt)
                    new_widget = None
                    if widget_class in (
                        "Select",
                        "SelectMultiple",
                        "ToggleSelectWidget",
                        "LazySelect",
                    ):
                        new_widget = new_widget_class(
                            form.fields[field].widget.attrs,
                            form.fields[field].widget.choices,
                        )
                    elif widget_class in (
                        "DateInput",
                        "DateTimeInput",
                        "TimeInput",
                    ):
                        new_widget = new_widget_class(
                            form.fields[field].widget.attrs,
                            form.fields[field].widget.format,
                        )
                        new_widget.supports_microseconds = form.fields[
                            field
                        ].widget.supports_microseconds
                    else:
                        new_widget = new_widget_class(
                            form.fields[field].widget.attrs
                        )
                    form.fields[field].widget = new_widget

            field_class_name = form.fields[field].__class__.__name__
            if field_class_name == "ModelChoiceField" and hasattr(
                settings, "ARCTIC_AUTOCOMPLETE"
            ):
                for key, values in settings.ARCTIC_AUTOCOMPLETE.items():
                    field_cls = ".".join(values[0].lower().split(".")[-2:])
                    if field_cls == str(
                        form.fields[field].queryset.model._meta
                    ):
                        url = reverse("arctic:autocomplete", args=[key, ""])
                        choices = ()
                        if form.instance.pk:
                            field_id = getattr(form.instance, field + "_id")
                            field_value = getattr(form.instance, field)
                            choices = ((field_id, field_value),)
                        form.fields[field].widget = SelectizeAutoComplete(
                            attrs=form.fields[field].widget.attrs,
                            choices=choices,
                            url=url,
                        )

            if self.readonly_fields and field in self.readonly_fields:
                form.fields[field].widget.attrs["readonly"] = True
        return form

    def get_form(self, form_class=None):
        if not self.form:
            self.form = super(FormMixin, self).get_form(form_class=None)
            try:
                self.form = self.update_form_fields(self.form)
            except AttributeError:
                pass
        return self.form

    def get_context_data(self, **kwargs):
        context = super(FormMixin, self).get_context_data(**kwargs)
        try:
            for i, formset in enumerate(context["inlines"]):
                try:
                    verbose_name = self.inlines[i].verbose_name
                except AttributeError:
                    verbose_name = formset.model._meta.verbose_name_plural
                setattr(context["inlines"][i], "verbose_name", verbose_name)
                extra = getattr(self.inlines[i], "inline_extra", 1)
                context["inlines"][i].extra = extra
                if hasattr(self.inlines[i], "sorting_field"):
                    setattr(
                        context["inlines"][i],
                        "sorting_field",
                        self.inlines[i].sorting_field,
                    )

                for j, form in enumerate(formset):
                    context["inlines"][i][j].fields = self.update_form_fields(
                        form
                    ).fields
        except KeyError:
            pass
        return context


class ListMixin(ModalMixin):
    template_name = "arctic/base_list.html"
    fields = None  # Which fields should be shown in listing
    ordering_fields = []  # Fields with ordering (subset of fields)
    field_links = {}
    field_classes = {}
    action_links = []  # "Action" links on item level. For example "Edit"
    tool_links = []  # Global links. For Example "Add object"
    default_ordering = []  # Default ordering, e.g. ['title', '-brand']
    search_fields = []
    # Simple search form if search_fields is defined
    simple_search_form_class = SimpleSearchForm
    advanced_search_form_class = None  # Custom form for advanced search
    _simple_search_form = None
    _advanced_search_form = None
    tool_links_icon = "fa-ellipsis-h"
    tool_links_collapse = 1
    max_embeded_list_items = 10  # when displaying a list in a column
    primary_key = "pk"
    sorting_field = None
    sorting_url = None

    @property
    def simple_search_form(self):
        return self._simple_search_form

    @property
    def advanced_search_form(self):
        return self._advanced_search_form

    def ordering_url(self, field_name):
        """
        Creates a url link for sorting the given field.

        The direction of sorting will be either ascending, if the field is not
        yet sorted, or the opposite of the current sorting if sorted.
        """
        path = self.request.path
        direction = ""
        query_params = self.request.GET.copy()
        ordering = self.request.GET.get("order", "").split(",")
        field = self._get_ordering_field_lookup(field_name)
        if not ordering:
            ordering = self.get_default_ordering()
        merged_ordering = list(ordering)  # copy the list

        for ordering_field in self.get_ordering_fields_lookups():
            if (ordering_field.lstrip("-") not in ordering) and (
                ("-" + ordering_field.lstrip("-")) not in ordering
            ):
                merged_ordering.append(ordering_field)

        new_ordering = []
        for item in merged_ordering:
            if item.lstrip("-") == field.lstrip("-"):
                if (item[0] == "-") or not (item in ordering):
                    if item in ordering:
                        direction = "desc"
                    new_ordering.insert(0, item.lstrip("-"))
                else:
                    direction = "asc"
                    new_ordering.insert(0, "-" + item)

        query_params["order"] = ",".join(new_ordering)

        return (path + "?" + query_params.urlencode(safe=","), direction)

    def get_fields(self, strip_labels=False):
        """
        Hook to dynamically change the fields that will be displayed
        """
        if self.fields == "__all__":
            return [f.name for f in self.model._meta.get_fields()]
        if strip_labels:
            return [
                f[0] if type(f) in (tuple, list) else f for f in self.fields
            ]
        return self.fields

    def get_ordering_fields(self):
        """
        Hook to dynamically change the fields that can be ordered
        """
        return self.ordering_fields

    def get_ordering_fields_lookups(self):
        """
        Getting real model fields to order by
        """
        ordering_field = []
        for field_name in self.get_ordering_fields():
            ordering_field.append(self._get_ordering_field_lookup(field_name))
        return ordering_field

    def _get_ordering_field_lookup(self, field_name):
        """
        get real model field to order by
        """
        field = field_name
        get_field = getattr(self, "get_%s_ordering_field" % field_name, None)
        if get_field:
            field = get_field()
        return field

    def get_tool_links_icon(self):
        return self.tool_links_icon

    def get_search_fields(self):
        """
        Hook to dynamically change the fields that can be searched
        """
        return self.search_fields

    def get_field_links(self):
        if not self.field_links:
            return {}
        else:
            allowed_field_links = {}
            for field, url in self.field_links.items():
                # check permission based on named_url
                view = view_from_url(url)
                if view.has_permission(self.request.user):
                    allowed_field_links[field] = url
                    self._extract_confirm_dialog(view, url)
            return allowed_field_links

    def get_field_classes(self, obj):
        field_classes = self.field_classes
        for field_name in field_classes.keys():
            get_field_name_classes = getattr(
                self, "get_%s_field_classes" % field_name, None
            )
            if get_field_name_classes:
                field_classes[field_name] = get_field_name_classes(obj)
        return field_classes

    def _get_field_actions(self, obj):
        all_actions = self.get_action_links()
        get_field_actions = getattr(self, "get_field_actions", None)
        if get_field_actions:
            field_actions = get_field_actions(obj)
            allowed_field_actions = self._get_allowed_field_actions(
                field_actions, all_actions
            )
        else:
            allowed_field_actions = all_actions
        if allowed_field_actions:
            actions = []
            for field_action in allowed_field_actions:
                actions.append(
                    {
                        "label": field_action["label"],
                        "icon": field_action["icon"],
                        "class": field_action["class"],
                        "url": self.in_modal(
                            reverse_url(
                                field_action["url"], obj, self.primary_key
                            )
                        ),
                        "modal": self.get_modal_link(field_action["url"], obj),
                        "attributes": field_action["attributes"],
                    }
                )
            return actions

    def _get_allowed_field_actions(self, field_actions, all_actions):
        allowed_urls = [a["url"] for a in all_actions]
        allowed_actions = []
        for action in field_actions:
            if action[1] in allowed_urls:
                allowed_actions.append(self._build_action_link(action))
        return allowed_actions

    def get_action_links(self):
        self._allowed_action_links = []
        if self.action_links:
            for link in self.action_links:
                url = named_url = link[1]
                if type(url) in (list, tuple):
                    named_url = url[0]
                # check permission based on named_url
                view = view_from_url(named_url)
                if view.has_permission(self.request.user):
                    self._allowed_action_links.append(
                        self._build_action_link(link)
                    )
                    self._extract_confirm_dialog(view, url)

        return self._allowed_action_links

    def _build_action_link(self, action_link):
        icon, attributes = None, None
        attributes_class = generate_id("action", action_link[0])
        if len(action_link) == 3:
            # icon can be 3-rd arg of link or specified inside inside dict with same index
            if isinstance(action_link[2], str):
                icon = action_link[2]
            elif isinstance(action_link[2], dict):
                icon = action_link[2].get("icon_class", None)
                attributes = action_link[2].get("attributes", None)
                if (
                    attributes
                    and attributes.get("class", None)
                    and isinstance(attributes.get("class", None), list)
                ):
                    attributes_class = " ".join(attributes.get("class"))
        return {
            "label": action_link[0],
            "url": action_link[1],
            "class": attributes_class,
            "icon": icon,
            "attributes": attributes,
        }

    def get_tool_links(self):
        if not self.tool_links:
            return []
        else:
            allowed_tool_links = []
            for link in self.tool_links:
                if (
                    (type(link[1]) in [tuple, list])
                    and callable(getattr(self, link[1][0], None))
                ) or callable(getattr(self, link[1], None)):
                    url = getattr(self, link[1][0])(*link[1][1:])
                    view = None
                else:
                    url = reverse_url(link[1], None)
                    view = view_from_url(link[1])
                if (view is None) or view.has_permission(self.request.user):
                    self._extract_confirm_dialog(view, link[1])
                    allowed_tool_link = {
                        "label": link[0],
                        "url": self.in_modal(url),
                        "style": "secondary",
                        "id": generate_id("tool-link", link[0]),
                        "modal": self.get_modal_link(link[1]),
                    }
                    if len(link) == 3:  # if an icon class is given
                        if type(link[2]) is str:
                            allowed_tool_link["icon"] = link[2]
                        elif type(link[2]) is dict:
                            allowed_tool_link.update(link[2])
                    allowed_tool_links.append(allowed_tool_link)
            return allowed_tool_links

    def get_simple_search_form_class(self):
        """
        Hook to dynamically change the simple search form
        """
        if not self.simple_search_form_class and self.get_search_fields():
            return SimpleSearchForm
        return self.simple_search_form_class

    def get_advanced_search_form_class(self):
        """
        Hook to dynamically change the advanced search form
        """
        return self.advanced_search_form_class

    def get_simple_search_form(self, data):
        if self.get_search_fields():
            self._simple_search_form = self.get_simple_search_form_class()(
                search_fields=self.get_search_fields(), data=data
            )
        else:
            self._simple_search_form = self.get_simple_search_form_class()(
                data=data
            )
        return self._simple_search_form

    def get_advanced_search_form(self, data):
        """
        Hook to dynamically change the advanced search form
        """
        if self.get_advanced_search_form_class():
            self._advanced_search_form = self.get_advanced_search_form_class()(
                data=data
            )
            return self._advanced_search_form

    @staticmethod
    def _field_is_m2m(m2m_fields_names, field):
        field_name = field.split("__")[0]
        for m2m_field in m2m_fields_names:
            if m2m_field in field_name:
                return m2m_field
        return False


class RoleAuthentication(object):
    """
    This class adds a role relation to the standard django auth user to add
    support for role based permissions in any class - usually a View.
    """

    ADMIN = "admin"
    permission_required = None

    def dispatch(self, request, *args, **kwargs):
        if not self.has_permission(request.user):
            raise PermissionDenied()
        return super(RoleAuthentication, self).dispatch(
            request, *args, **kwargs
        )

    @classmethod
    def sync(cls):
        """
        Save all the roles defined in the settings that are not yet in the db
        this is needed to create a foreign key relation between a user and a
        role. Roles that are no longer specified in settings are set as
        inactive.
        """
        try:
            settings_roles = set(settings.ARCTIC_ROLES.keys())
        except AttributeError:
            settings_roles = set()

        saved_roles = set(Role.objects.values_list("name", flat=True))
        unsaved_roles = settings_roles - saved_roles
        unused_roles = saved_roles - settings_roles - set([cls.ADMIN])

        # ensure that admin is not defined in settings
        if cls.ADMIN in settings_roles:
            raise ImproperlyConfigured(
                '"' + cls.ADMIN + '" role is reserved '
                "and cannot be defined in settings"
            )

        # ensure that admin exists in the database
        if cls.ADMIN not in saved_roles:
            Role(name=cls.ADMIN, is_active=True).save()

        # check if the role defined in settings already exists in the database
        # and if it does ensure it is enabled.
        for role in saved_roles:
            if role in settings_roles:
                saved_role = Role.objects.get(name=role)
                if not saved_role.is_active:
                    saved_role.is_active = True
                    saved_role.save()

        for role in unsaved_roles:
            Role(name=role).save()

        for role in unused_roles:
            unused_role = Role.objects.get(name=role)
            unused_role.is_active = False
            unused_role.save()

    @classmethod
    def get_permission_required(cls):
        """
        Get permission required property.
        Must return an iterable.
        """
        if cls.permission_required is None:
            raise ImproperlyConfigured(
                "{0} is missing the permission_required attribute. "
                "Define {0}.permission_required, or override "
                "{0}.get_permission_required().".format(cls.__name__)
            )
        if isinstance(cls.permission_required, str):
            if cls.permission_required != "":
                perms = (cls.permission_required,)
            else:
                perms = ()
        else:
            perms = cls.permission_required

        return perms

    @classmethod
    def has_permission(cls, user):
        """
        We override this method to customize the way permissions are checked.
        Using our roles to check permissions.
        """
        # no login is needed, so its always fine
        if not cls.requires_login:
            return True

        # if user is somehow not logged in
        if not user.is_authenticated:
            return False

        # attribute permission_required is mandatory, returns tuple
        perms = cls.get_permission_required()
        # if perms are defined and empty, we skip checking
        if not perms:
            return True

        # get role of user, skip admin role
        role = user.urole.role.name
        if role == cls.ADMIN:
            return True

        # check if at least one permissions is valid
        for permission in perms:
            if cls.check_permission(role, permission):
                return True

        # permission denied
        return False

    @classmethod
    def check_permission(cls, role, permission):
        """
        Check if role contains permission
        """
        result = permission in settings.ARCTIC_ROLES[role]
        # will try to call a method with the same name as the permission
        # to enable an object level permission check.
        if result:
            try:
                return getattr(cls, permission)(role)
            except AttributeError:
                pass
        return result


class FormMediaMixin(object):
    """
    Gathers media assets defined in forms
    """

    @property
    def media(self):
        # Why not simply call super? Just
        # to have custom media defined at the bottom,
        # since order matters here.
        media = self._get_common_media()
        media += self._get_view_media()
        media += self._get_form_media()
        media += self.get_media_assets()

        return media

    def _get_form_media(self):
        form = self.get_form()
        return getattr(form, "media", None)



from django.utils.translation import gettext_lazy as _

ARCTIC_SITE_NAME = _("Arctic Example")
ARCTIC_FORM_DISPLAY = "float-label"

# Menu format:
# (('menu label', 'named url', 'optional icon class',
#   (optional submenu), (optional related_urls)) )
ARCTIC_MENU = (
    (_("Dashboard"), "index", "fa-dashboard"),
    (
        _("Articles"),
        None,
        "fa-file-text-o",
        (
            (
                _("List"),
                "articles:list",
                ("articles:detail", "articles:delete"),
            ),
            (_("Create"), "articles:create"),
        ),
    ),
    (
        _("Categories"),
        None,
        "fa-sitemap",
        (
            (
                _("List"),
                "articles:category-list",
                ("articles:category-detail", "articles:category-delete"),
            ),
            (_("Create"), "articles:category-create"),
        ),
    ),
    (
        _("Tags"),
        None,
        "fa-tags",
        (
            (
                _("List"),
                "articles:tag-list",
                ("articles:tag-detail", "articles:tag-delete"),
            ),
            (_("Create"), "articles:tag-create"),
        ),
    ),
    (
        _("Users"),
        None,
        "fa-user",
        (
            (_("List"), "users:list", ("users:detail",)),
            (_("Create"), "users:create"),
        ),
    ),
    (_("Countries"), "countries-list", "fa-globe"),
)

ARCTIC_USER_ROLE_MODEL = "arctic.UserRole"
ARCTIC_ROLE_MODEL = "arctic.Role"

ARCTIC_ROLES = {
    "editor": (
        "view_dashboard",
        "view_article",
        "change_article",
        "add_article",
    )
}

ARCTIC_AUTOCOMPLETE = {"categories": ("articles.Category", "name")}

ARCTIC_DARK_MODE = True
# ARCTIC_DARK_MODE = 'toggle'

# ARCTIC_SIDEBAR_BACKGROUND = '#BF1D1D'
# ARCTIC_HIGHLIGHT_COLOR = '#9ec3d5'

# ARCTIC_SIDEBAR_BACKGROUND = '#212050'
# ARCTIC_HIGHLIGHT_COLOR = '#76a2cb'

# ARCTIC_SIDEBAR_BACKGROUND_DARK = '#FF0000'
# ARCTIC_SIDEBAR_COLOR_DARK = '#00FF00'
# ARCTIC_SIDEBAR_ALT_COLOR_DARK = '#0000FF'



# Generated by Django 2.1.4 on 2019-02-26 12:42

import articles.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('articles', '0005_auto_20180625_1539'),
    ]

    operations = [
        migrations.AddField(
            model_name='image',
            name='tags',
            field=models.ManyToManyField(to='articles.Tag'),
        ),
        migrations.AddField(
            model_name='image',
            name='updated_at',
            field=models.DateTimeField(auto_now=True, null=True),
        ),
        migrations.AlterField(
            model_name='image',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to=articles.models.filename),
        ),
    ]



XXXXXXX
XXXXXXXXXXX
XXXXXXXXX
XXXXXXXXX
XXXXXXXX
XXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXX
XXXXXXXXXXXXXXX
XXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


# Generated by Django 2.1.4 on 2019-03-01 09:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('articles', '0006_auto_20190226_1342'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='image',
            name='updated_at',
        ),
        migrations.AddField(
            model_name='image',
            name='updated',
            field=models.DateTimeField(null=True),
        ),
    ]



import pytest
from django.urls import reverse

from tests.factories import ArticleFactory


@pytest.mark.django_db
class TestOrderingView(object):
    """
    Test ordering on ListView
    """
    def _request(self, admin_client):
        response = admin_client.get(reverse('articles:list'),
                                    data={'order': '-category__name'})
        assert response.status_code == 200
        return response

    def test_virtual_ordering_field(self, admin_client):
        """
        ordering by 'virtual ordering field'
        """
        ArticleFactory(title='Article 0')
        ArticleFactory(title='Article 1')
        response = self._request(admin_client)
        lst_headers = response.context_data['list_header']
        lst_items = response.context_data['list_items']
        assert lst_headers[3]['order_url'] == '/articles/?order=category__name'
        assert lst_headers[3]['order_direction'] == 'desc'
        assert len(lst_items) == 2
        assert lst_items[0]['fields'][0]['value'] == 'Article 1'
        assert lst_items[1]['fields'][0]['value'] == 'Article 0'



from django.apps import apps as django_apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def get_role_model():
    """
    Returns the Role model that is active in this project.
    """
    app_model = getattr(settings, "ARCTIC_ROLE_MODEL", "arctic.Role")

    try:
        return django_apps.get_model(app_model)
    except ValueError:
        raise ImproperlyConfigured(
            "ARCTIC_ROLE_MODEL must be of the " "form 'app_label.model_name'"
        )
    except LookupError:
        raise ImproperlyConfigured(
            "ARCTIC_ROLE_MODEL refers to model '%s' that has not been "
            "installed" % settings.ARCTIC_ROLE_MODEL
        )


def get_user_role_model():
    """
    Returns the UserRole model that is active in this project.
    """
    app_model = getattr(settings, "ARCTIC_USER_ROLE_MODEL", "arctic.UserRole")

    try:
        return django_apps.get_model(app_model)
    except ValueError:
        raise ImproperlyConfigured(
            "ARCTIC_USER_ROLE_MODEL must be of the "
            "form 'app_label.model_name'"
        )
    except LookupError:
        raise ImproperlyConfigured(
            "ARCTIC_USER_ROLE_MODEL refers to model '%s' that has not been "
            "installed" % settings.ARCTIC_USER_ROLE_MODEL
        )



# -*-*- encoding: utf-8 -*-*-

from django.db import models


class {{ camel_case_app_name }}(models.Model):
    # Add DB fields

    # Optional add __str__
    # def __str__(self):
        # return self.name



from django.conf.urls import url

from . import views

app_name = "{{ app_name }}"

urlpatterns = [url(r"^$", views.{{ camel_case_app_name }}ListView.as_view(), name="list"),
               url(r"^create/$", views.{{ camel_case_app_name }}CreateView.as_view(), name="create"),
               url(r"^(?P<pk>\d+)/$", views.{{ camel_case_app_name }}UpdateView.as_view(), name="detail"),
               url(r"^(?P<pk>\d+)/delete/$", views.{{ camel_case_app_name }}DeleteView.as_view(), name="delete"),
               ]



from datetime import datetime

import django
from django.forms.widgets import (
    ClearableFileInput,
    DateInput,
    DateTimeInput,
    Select,
    SelectMultiple,
    TextInput,
    TimeInput,
    Input,
    CheckboxSelectMultiple,
    RadioSelect,
)
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe


class StyledSelect(Select):
    def render(self, name, value, attrs=None, renderer=None):
        try:
            select_render = super(StyledSelect, self).render(
                name, value, attrs, renderer
            )
        except TypeError:  # versions older than Django 1.11
            select_render = super(StyledSelect, self).render(
                name, value, attrs
            )

        return mark_safe(
            '<div class="styled-select">{}</div>'.format(select_render)
        )


class Selectize(Select):
    def __init__(self, attrs={}, choices=()):
        attrs["js-selectize"] = True
        super(Selectize, self).__init__(attrs, choices)


class SelectizeMultiple(SelectMultiple):
    def __init__(self, attrs={}, choices=()):
        attrs["js-selectize-multiple"] = True
        super(SelectizeMultiple, self).__init__(attrs, choices)


class SelectizeAutoComplete(Select):
    def __init__(self, url, attrs={}, choices=()):
        attrs["js-selectize-autocomplete"] = True
        attrs["data-url"] = url
        super(SelectizeAutoComplete, self).__init__(attrs, choices)


class PickerFormatMixin(Input):
    """
    Handle formatting of widget input value

    Attributes:
        display_format(str): string that will
        be used to format input value before render

        widget_attribute_key(str): represents attribute name
        to which formatted input value will be assigned
    """

    display_format = None
    widget_attribute_key = None

    def get_context(self, name, value, attrs):
        context = super(PickerFormatMixin, self).get_context(
            name, value, attrs
        )
        if isinstance(value, datetime):
            value = value.strftime(self.display_format)
        context["widget"]["attrs"][self.widget_attribute_key] = value
        return context


class DateTimePickerInput(PickerFormatMixin, DateTimeInput):
    def __init__(self, attrs={}, format=None):
        attrs["js-datetimepicker"] = True
        self.display_format = "%m/%d/%Y %I:%M %p"
        self.widget_attribute_key = "data-datetime"
        super(DateTimePickerInput, self).__init__(attrs, format)


class DatePickerInput(PickerFormatMixin, DateInput):
    def __init__(self, attrs={}, format=None):
        attrs["js-datepicker"] = True
        self.display_format = "%m/%d/%Y"
        self.widget_attribute_key = "data-date"
        super(DatePickerInput, self).__init__(attrs, format)


class TimePickerInput(PickerFormatMixin, TimeInput):
    def __init__(self, attrs={}, format=None):
        attrs["js-timepicker"] = True
        self.display_format = "%I:%M %p"
        self.widget_attribute_key = "data-time"
        super(TimePickerInput, self).__init__(attrs, format)


class QuickFiltersSelectMixin(object):
    template_name = "arctic/widgets/quick_filters_select.html"

    def get_context(self, name, value, attrs=None, *args, **kwargs):
        if django.VERSION >= (1, 11):
            return super(QuickFiltersSelectMixin, self).get_context(
                name, value, attrs
            )
        else:
            # django 1.10 doesn't support optgroups
            # and render choices in method
            context = {"widget": self, "attrs": attrs}
            optgroups = []
            for val, label in self.choices:
                option = {
                    "name": name,
                    "value": val,
                    "selected": val in value,
                    "label": label,
                }
                optgroups.append((None, [option], None))
            context["widget"].optgroups = optgroups
        return context

    def render(self, name, value, attrs=None, renderer=None):
        """For django 1.10 compatibility"""
        if django.VERSION >= (1, 11):
            return super(QuickFiltersSelectMixin, self).render(
                name, value, attrs
            )

        t = render_to_string(
            template_name=self.template_name,
            context=self.get_context(name, value, attrs),
        )
        return mark_safe(t)


class QuickFiltersSelect(QuickFiltersSelectMixin, RadioSelect):
    """
    This widget is used when you want select only one active filter
    """


class QuickFiltersSelectMultiple(
    QuickFiltersSelectMixin, CheckboxSelectMultiple
):
    """
    This widget is used to be able to have a more than one active filters
    """

    def __init__(self, attrs=None, **kwargs):
        attrs = attrs or {}
        attrs["select_multiple"] = True
        super().__init__(attrs, **kwargs)


class SearchInput(TextInput):
    """
    Widget used in the inline search field on top of ListViews
    """

    template_name = "arctic/widgets/search_input.html"

    def render(self, name, value, attrs=None, renderer=None):
        """For django 1.10 compatibility"""
        if django.VERSION >= (1, 11):
            return super(SearchInput, self).render(name, value, attrs)

        t = render_to_string(
            template_name=self.template_name,
            context=self.get_context(name, value, attrs),
        )
        return mark_safe(t)


class BetterFileInput(ClearableFileInput):
    """
    File input replacement with Image preview
    """

    template_name = "arctic/widgets/file_input.html"

    def render(self, name, value, attrs=None, renderer=None):
        """For django 1.10 compatibility"""
        if django.VERSION >= (1, 11):
            return super(BetterFileInput, self).render(name, value, attrs)

        t = render_to_string(
            template_name=self.template_name,
            context=self.get_context(name, value, attrs),
        )
        return mark_safe(t)



from django import forms

from .models import {{ camel_case_app_name }}


class {{ camel_case_app_name }}Form(forms.ModelForm):
    class Meta:
        fields = '__all__'
        model = {{ camel_case_app_name }}



from django.contrib.auth import get_user_model
from django.db.models.signals import post_save

from arctic.loading import get_role_model, get_user_role_model

from .mixins import RoleAuthentication

User = get_user_model()
Role = get_role_model()
UserRole = get_user_role_model()


def superuser_post_save(sender, instance, created, **kwargs):
    if created and instance.is_superuser:
        RoleAuthentication.sync()
        admin = Role.objects.get(name="admin")
        try:
            UserRole.objects.get(user=instance, role=admin)
        except UserRole.DoesNotExist:
            UserRole(user=instance, role=admin).save()


post_save.connect(superuser_post_save, sender=User)



# Generated by Django 2.0.6 on 2018-06-25 13:39

import articles.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('articles', '0004_image'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='image',
            options={'ordering': ['order'], 'verbose_name': 'Image', 'verbose_name_plural': 'Images'},
        ),
        migrations.AlterField(
            model_name='image',
            name='image',
            field=models.ImageField(null=True, upload_to=articles.models.filename),
        ),
    ]



# -*-*- encoding: utf-8 -*-*-
from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.static import static

from arctic.generics import LoginView, LogoutView
from arctic.views import handler400, handler403, handler404, handler500  # noqa

from countries.views import CountryAPIView, CountryListView
from dashboard.views import DashboardView

urlpatterns = [
    url(r"^$", DashboardView.as_view(), name="index"),
    url(r"^login/$", LoginView.as_view(), name="login"),
    url(r"^logout/$", LogoutView.as_view(), name="logout"),
    url(r"^articles/", include("articles.urls", "articles")),
    url(r"^users/", include("arctic.users.urls", namespace="users")),
    url(r"^countries/$", CountryListView.as_view(), name="countries-list"),
    url(r"^countries-api/$", CountryAPIView.as_view(), name="countries-api"),
    url(r"^arctic/", include("arctic.urls", namespace="arctic")),
]


if settings.DEBUG:
    try:
        import debug_toolbar

        urlpatterns = [
            url(r"^__debug__/", include(debug_toolbar.urls))
        ] + urlpatterns
    except ImportError:
        pass

    urlpatterns += static(
        settings.MEDIA_URL, document_root=settings.MEDIA_ROOT
    )



from django.db.models import Q
# from django.http import HttpRequest
# from django.template import Context, Template
# from bs4 import BeautifulSoup

from arctic.forms import SimpleSearchForm
from arctic.widgets import QuickFiltersSelect


def test_simple_search_form():
    data = {'search': None}
    search_fields = ['name']

    form = SimpleSearchForm(search_fields=search_fields, data=data)
    search_filter = form.get_search_filter()

    # Empty search value, should return empty filter
    assert str(search_filter) == str(Q())

    data = {'search': 'test'}
    form = SimpleSearchForm(search_fields=search_fields, data=data)
    search_filter = form.get_search_filter()

    # Filled in search value, should return icontains filter
    assert str(search_filter) == str(Q(('name__icontains', 'test')))


def test_filters_widget_attr():
    widget = QuickFiltersSelect
    assert widget.template_name == 'arctic/widgets/quick_filters_select.html'


# def test_form_rendering_with_request_get_args():
#     request = HttpRequest()
#     request.GET['search'] = 'cats'
#     request.GET['my_filters'] = 'published'

#     form = FiltersForm(data=request.GET)
#     assert form.is_valid()

#     template = Template('{{ form }}').render(Context({"form": form}))

#     soup = BeautifulSoup(template, 'html.parser')

#     filters_inputs = soup.find_all('input', {'type': 'checkbox'})
#     filters_buttons = soup.find_all('label', {'class': 'btn'})

#     assert len(filters_inputs) == len(form.FILTER_BUTTONS)
#     assert len(filters_buttons) == len(form.FILTER_BUTTONS)

#     for inp, btn, choice in zip(
#             filters_inputs,
#             filters_buttons,
#             form.FILTER_BUTTONS):
#         assert btn.text.strip() == choice[1]
#         assert inp.get('value') == choice[0]
#         assert inp.get('name') == 'my_filters'

#     # check if we marked selected filter
#     assert filters_inputs[0].attrs.get('checked') == ''


# def test_filters_form_rendering_select_one():
#     request = HttpRequest()
#     request.GET['search'] = 'cats'
#     request.GET['my_filters'] = 'published'

#     form = FiltersFormSelectOne(data=request.GET)
#     assert form.is_valid()

#     template = Template('{{ form }}').render(Context({"form": form}))

#     soup = BeautifulSoup(template, 'html.parser')
#     filters_inputs = soup.find_all('input', {'type': 'checkbox'})

#     assert filters_inputs[0].attrs.get('checked') is None



from django.conf.urls import url

from articles.views import (
    ArticleCreateView,
    ArticleDeleteView,
    ArticleListView,
    ArticleUpdateView,
    CategoryArticlesListView,
    CategoryCreateView,
    CategoryDeleteView,
    CategoryListView,
    CategoryUpdateView,
    TagCreateView,
    TagDeleteView,
    TagListView,
    TagUpdateView,
)

app_name = "articles"

urlpatterns = [
    url(r"^$", ArticleListView.as_view(), name="list"),
    url(r"^create/$", ArticleCreateView.as_view(), name="create"),
    url(r"^(?P<pk>\d+)/$", ArticleUpdateView.as_view(), name="detail"),
    url(r"^(?P<pk>\d+)/delete/$", ArticleDeleteView.as_view(), name="delete"),
    url(r"^category/$", CategoryListView.as_view(), name="category-list"),
    url(
        r"^category/create/$",
        CategoryCreateView.as_view(),
        name="category-create",
    ),
    url(
        r"^category/(?P<pk>\d+)/$",
        CategoryUpdateView.as_view(),
        name="category-detail",
    ),
    url(
        r"^category/articles/(?P<pk>\d+)/$",
        CategoryArticlesListView.as_view(),
        name="category-articles-list",
    ),
    url(
        r"^category/(?P<pk>\d+)/delete/$",
        CategoryDeleteView.as_view(),
        name="category-delete",
    ),
    url(r"^tags/$", TagListView.as_view(), name="tag-list"),
    url(r"^tags/create/$", TagCreateView.as_view(), name="tag-create"),
    url(r"^tags/(?P<pk>\d+)/$", TagUpdateView.as_view(), name="tag-detail"),
    url(
        r"^tags/(?P<pk>\d+)/delete/$",
        TagDeleteView.as_view(),
        name="tag-delete",
    ),
]



# options are 'float-label', 'stacked' or 'tabular'
ARCTIC_FORM_DISPLAY = "float-label"

ARCTIC_SITE_LOGO = "arctic/dist/assets/img/arctic_logo.svg"

ARCTIC_SITE_NAME = "Arctic Site Name"

ARCTIC_WIDGET_OVERLOADS = {
    "DateInput": "arctic.widgets.DatePickerInput",
    "DateTimeInput": "arctic.widgets.DateTimePickerInput",
    "ClearableFileInput": "arctic.widgets.BetterFileInput",
    "FileInput": "arctic.widgets.BetterFileInput",
    "TimeInput": "arctic.widgets.TimePickerInput",
    "Select": "arctic.widgets.StyledSelect",
    "SelectMultiple": "arctic.widgets.SelectizeMultiple",
    "MultipleChoiceField": "arctic.widgets.Selectize",
}

ARCTIC_PAGINATION = {"show_label": True, "show_first_last": True, "range": 5}

ARCTIC_PAGINATION_TEMPLATE = "arctic/partials/pagination.html"



# TODO: Move extra views into our own views. So we can override where needed
from extra_views import InlineFormSetFactory

from .models import Article, Image


class ArticleInline(InlineFormSetFactory):
    model = Article
    fields = "__all__"


class TagsInline(InlineFormSetFactory):
    model = Article.tags.through
    fields = "__all__"


class ImagesInline(InlineFormSetFactory):
    model = Image
    fields = "__all__"
    layout = {"blah": "bah"}
    sorting_field = "order"
    inline_extra = 1
    factory_kwargs = {"max_num": 7}



import operator

from arctic.widgets import SearchInput
from django import forms
from functools import reduce

from django.db.models import Q


class SimpleSearchForm(forms.Form):
    search = forms.CharField(max_length=50, widget=SearchInput, required=False)

    def __init__(self, search_fields=[], *args, **kwargs):
        super(SimpleSearchForm, self).__init__(*args, **kwargs)
        self.search_fields = search_fields

    def get_search_filter(self):
        value = self.data.get("search")
        if not value:
            return Q()

        q_list = []
        for field_name in self.search_fields:
            q_list.append(Q(**{field_name + "__icontains": value}))

        return reduce(operator.or_, q_list)



# -*-*- encoding: utf-8 -*-*-
from __future__ import unicode_literals

from datetime import datetime

from django.db import models
from django.utils.translation import gettext_lazy as _


class Article(models.Model):
    created_at = models.DateTimeField(auto_now=True)
    updated_at = models.DateTimeField()
    title = models.CharField(_("Title"), max_length=255, null=False)
    description = models.TextField(_("Description"), blank=True, null=False)
    published = models.BooleanField(_("Published"), default=False)
    category = models.ForeignKey(
        "articles.Category",
        verbose_name=_("Category"),
        on_delete=models.CASCADE,
    )
    tags = models.ManyToManyField("articles.Tag")
    order = models.IntegerField(_("Order"), blank=True, null=True)

    def __str__(self):
        return self.title

    class Meta:
        ordering = ["id"]


class Category(models.Model):
    name = models.CharField(
        _("Name"), max_length=255, null=False, blank=False, unique=True
    )
    order = models.IntegerField(_("Order"), blank=True, null=True)

    def __str__(self):
        return self.name


class Tag(models.Model):
    term = models.CharField(
        _("Term"), max_length=255, null=False, blank=False, unique=True
    )

    def __str__(self):
        return self.term


def filename(instance, filename):
    return "uploads/" + datetime.now().strftime("%Y%m%d%H%M%S%f") + ".jpg"


class Image(models.Model):
    title = models.CharField(max_length=255, null=True, blank=True)
    image = models.ImageField(upload_to=filename, null=True, blank=True)
    article = models.ForeignKey(Article, on_delete=models.CASCADE)
    order = models.IntegerField(null=True, blank=True)
    tags = models.ManyToManyField("articles.Tag")
    updated = models.DateTimeField(null=True)

    class Meta:
        ordering = ["order"]
        verbose_name = "Image"
        verbose_name_plural = "Images"

    def __str__(self):
        return self.title



#!/usr/bin/env python
from __future__ import absolute_import, print_function, unicode_literals

import os
import stat
from optparse import OptionParser

from django.core.management import ManagementUtility


class bcolors:
    """
    ANSI escape sequences for terminal colors
    """

    HEADER = "\033[95m"
    OKBLUE = "\033[94m"
    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    ENDC = "\033[0m"
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"


def create_project(parser, options, args):
    # Validate args
    if len(args) < 2:
        parser.error("Please specify a name for your Arctic installation")
    elif len(args) > 3:
        parser.error("Too many arguments")

    project_name = args[1]
    try:
        dest_dir = args[2]
    except IndexError:
        dest_dir = ""

    # Make sure given name is not already in use by another
    # python package/module.
    try:
        __import__(project_name)
    except ImportError:
        pass
    else:
        parser.error(
            '"{}" conflicts with the name of an existing '
            "Python module and cannot be used as a project "
            "name. Please try another name.".format(project_name)
        )

    print("Creating an Arctic project named {}".format(project_name))

    # Create the project from the Arctic template using startapp

    # First find the path to Arctic
    import arctic

    arctic_path = os.path.dirname(arctic.__file__)
    template_path = os.path.join(arctic_path, "project_template/start")

    # Call django-admin startproject
    utility_args = [
        "django-admin.py",
        "startproject",
        "--template=" + template_path,
        "--ext=html,rst",
        project_name,
    ]

    if dest_dir:
        utility_args.append(dest_dir)

    utility = ManagementUtility(utility_args)
    utility.execute()

    # add execute permission to manage.py, somehow it gets lost on the way
    manage_py = os.path.join(dest_dir or project_name, "manage.py")
    st = os.stat(manage_py)
    os.chmod(
        manage_py, st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
    )

    print(
        "Congratulations! {0} has been created.\n"
        "The next steps are:\n"
        "- In config/settings.py change the database settings (if needed).\n"
        "- Run database migrations: {0}/manage.py migrate.\n"
        "- Create an admin user: {0}/manage.py createsuperuser.\n"
        "- Finally run the project: {0}/manage.py runserver.\n".format(
            project_name
        )
    )


def create_app(parser, options, args):
    # Validate args
    if len(args) < 2:
        parser.error("Please specify a name for your app")
    elif len(args) > 3:
        parser.error("Too many arguments")

    app_name = args[1].lower()

    try:
        dest_dir = args[2]
    except IndexError:
        dest_dir = ""

    # Make sure given name is not already in use by another
    # python package/module.
    try:
        __import__(app_name)
    except ImportError:
        pass
    else:
        parser.error(
            '"{}" conflicts with the name of an existing '
            "Python module and cannot be used as an app "
            "name. Please try another name.".format(app_name)
        )

    print(
        (
            bcolors.HEADER + "Creating an App named {}" + bcolors.ENDC + "\n"
        ).format(app_name)
    )

    # First find the path to Arctic
    import arctic

    arctic_path = os.path.dirname(arctic.__file__)
    template_path = os.path.join(arctic_path, "project_template/app")

    # Call django-admin starrtapp
    utility_args = [
        "django-admin.py",
        "startapp",
        "--template=" + template_path,
        app_name,
    ]

    if dest_dir:
        utility_args.append(dest_dir)

    utility = ManagementUtility(utility_args)
    utility.execute()

    print(
        (
            "Congratulations! {0} folder has been created it contains the "
            "following structure.\n\n" + bcolors.OKBLUE + " -{0}\n"
            " ---__init__.py\n"
            " ---apps.py\n"
            " ---forms.py\n"
            " ---models.py\n"
            " ---urls.py\n"
            " ---views.py\n\n" + bcolors.ENDC + "The next steps are:\n\n"
            "  Add the app name to "
            + bcolors.UNDERLINE
            + "INSTALLED_APPS"
            + bcolors.ENDC
            + " in the settings.py\n"  # NOQA
            + bcolors.OKGREEN
            + '"{0}",'
            + bcolors.ENDC
            + "\n"
            "  Add the app name and path to "
            + bcolors.UNDERLINE
            + "ARCTIC_MENU"
            + bcolors.ENDC
            + " in the settings.py\n"  # NOQA
            + bcolors.OKGREEN
            + '("{1}", "{0}:list", "fa-folder"),'
            + bcolors.ENDC
            + "\n"  # NOQA
            "  Add the urls to config/urls.py.\n"
            + bcolors.OKGREEN
            + 'url(r"^{0}/", include("{0}.urls", "{0}")),'
            + bcolors.ENDC
            + "\n"  # NOQA
            "  Add fields in the models.py file\n"
            "- Run "
            + bcolors.OKGREEN
            + "./manage.py makemigrations {0}"
            + bcolors.ENDC
            + "\n"  # NOQA
            "- Run "
            + bcolors.OKGREEN
            + "./manage.py migrate"
            + bcolors.ENDC
            + "\n\n"  # NOQA
            "The "
            + bcolors.BOLD
            + "forms.py"
            + bcolors.ENDC
            + " has a form with all the fields in the model and \n"  # NOQA
            "the "
            + bcolors.BOLD
            + "views.py"
            + bcolors.ENDC
            + " contains views for list, create, edit and delete. \n"  # NOQA
            "All of then can be tweaked to better satisfy the needs of the "
            "project/app\n"
        ).format(app_name, app_name.capitalize())
    )


COMMANDS = {
    "start": create_project,
    "createapp": create_app,
}


def main():
    # Parse options
    parser = OptionParser(
        usage="Usage: arctic start project_name [directory]"
        "Usage: arctic createapp appname [directory]"
    )
    (options, args) = parser.parse_args()

    # Find command
    try:
        command = args[0]
    except IndexError:
        parser.print_help()
        return

    if command in COMMANDS:
        COMMANDS[command](parser, options, args)
    else:
        parser.error("Unrecognised command: " + command)


if __name__ == "__main__":
    main()
